
(require 'auto-complete)


(defvar lig-cb-file nil
  "Current CB temp file-name")

(defvar lig-ac-string nil
  "Auto-complete string.")

(defun lig-set-method-string ()
  (setq lig-ac-string
	(concat "method_name ^" ac-prefix
		"\nunadd class\nadd class "
		"\n")
))

(defun lig-set-class-string ()
  (setq lig-ac-string
	(concat "method_name "
		"\nunadd class\nadd class ^" ac-prefix
		"\n")
))

(defvar lig-ac-cb-methods-buffer " *AC CB methods*"
  "Auto-complete CB buffer.")

(defvar lig-ac-cb-classes-buffer " *AC CB classes*"
  "Auto-complete CB buffer.")

(defface ac-method-candidate-face
   '((t (:background "SlateGray3" :foreground "black" )))
   "Face of Magik method candidate."
   :group 'auto-complete)

(defface ac-method-selection-face
   '((t (:background "SteelBlue4" :foreground "white" )))
   "Face of the Magik method selected candidate."
   :group 'auto-complete)

(defface ac-class-candidate-face
   '((t (:background "SlateGray2" :foreground "black" )))
   "Face of Magik class candidate."
   :group 'auto-complete)

(defface ac-class-selection-face
   '((t (:background "SteelBlue4" :foreground "white" )))
   "Face of the Magik class selected candidate."
   :group 'auto-complete)

(defvar ac-source-magik-methods
  '((candidates . lig-cb-send-method-string)
    (limit . 5)
    (candidate-face . ac-method-candidate-face)
    (selection-face . ac-method-selection-face)
    )
  "Source for Magik Methods.")

(defvar ac-source-magik-classes
  '((candidates . lig-cb-send-class-string)
    (limit . 5)
    (candidate-face . ac-class-candidate-face)
    (selection-face . ac-class-selection-face)
    )
  "Source for Magik Classes.")

(defvar lig-ac-complete nil
  "Auto-complete in use.")

(defvar lig-ac-cb-buffer nil
  "Auto-complete CB buffer.")

(defvar lig-cb-process nil
  "CB process.")



(defun lig-cb-send-method-string ()
 (if (cb-is-running (get-buffer "*cb**gis*"))
     (progn
       (let ((lig-cb-process (cb-process (get-buffer-create "*cb**gis*")))
	     candidates)

       (set-process-filter lig-cb-process 'lig-ac-cb-methods-filter)

       (process-send-string lig-cb-process (lig-set-method-string))
       (process-send-string lig-cb-process (concat "override_flags\n"
						   "dont_show_classes\n"
						   "local_only\n"
						   "print_curr_methods\n"
						   ))

       (save-excursion
	 (setq lig-ac-cb-buffer (get-buffer-create lig-ac-cb-methods-buffer ))
	 (set-buffer lig-ac-cb-buffer)
	 (setq candidates (ac-candidate-words-in-cb-buffer lig-ac-cb-buffer))
	 ); save-excursion
       ; reset CB mode-line
       (lig-reset-cb)

       candidates)))
)

(defun lig-cb-send-class-string ()
 (if (cb-is-running (get-buffer "*cb**gis*"))
     (progn
       (let ((lig-cb-process (cb-process (get-buffer-create "*cb**gis*")))
	     candidates)

       (set-process-filter lig-cb-process 'lig-ac-cb-classes-filter)

       (process-send-string (cb-process (get-buffer-create "*cb**gis*")) (lig-set-class-string))
       (process-send-string (cb-process (get-buffer-create "*cb**gis*")) (concat "override_flags\n"
									  "dont_show_method_names\n"
									  "local_only\n"
									  "print_curr_methods\n"
									  ))
       (save-excursion
	 (setq lig-ac-cb-buffer (get-buffer-create lig-ac-cb-classes-buffer ))
	 (set-buffer lig-ac-cb-buffer)
	 (setq candidates (ac-candidate-words-in-cb-buffer lig-ac-cb-buffer))
       );save-excursion
        ; reset CB mode-line
       (lig-reset-cb)

       candidates)))
)

(defun lig-reset-cb ()

  (sleep-for 0.1)
  (set-process-filter (cb-process (get-buffer-create "*cb**gis*")) 'cb-filter)
  
  (process-send-string (cb-process (get-buffer-create "*cb**gis*")) (concat "reset\n"))
  
  (save-excursion
    (set-buffer (get-buffer-create "*cb**gis*"))
    ;; since we just reseted cb - need to make all cosmetic changes 
    (cb-set-method-and-class "" "")
    (cb-set-topic "override-flags" nil)
    (cb-redraw-modeline)
    (cb-print-curr-methods)
    ); save-excursion
)


(defun lig-ac-cb-methods-filter (p s)
  "Process data coming back from the C."
  (save-match-data
    (let* ((b (get-buffer-create lig-ac-cb-methods-buffer ))
	   jump-str)
      (set-buffer b)
      (setq cb-filter-str (concat cb-filter-str s))
      (if (string-match "\C-e" cb-filter-str) (lig-cb-read-methods p b))

     )))

(defun lig-ac-cb-classes-filter (p s)
  "Process data coming back from the C."
  (save-match-data
    (let* ((b (get-buffer-create lig-ac-cb-classes-buffer ))
	   jump-str)
      (set-buffer b)
      (setq cb-filter-str (concat cb-filter-str s))
      (if (string-match "\C-e" cb-filter-str) (lig-cb-read-methods p b))

     )))


(defun lig-cb-read-methods (p buf)
  "Deal with a C-e or a C-u char coming back from the C by loading
from \"/tmp\" into the main cb buffer.  Be careful to maintain the
position in the listing.  Also extract the number-of-methods from
the last line of the file, and put it in the global `cb-n-methods-str'.
"
      (let ((coding-system-for-read cb-coding-system))

	(set-buffer buf)
	(erase-buffer)
	(insert-file-contents (cb-temp-file-name p)))
      )


(defun ac-candidate-words-in-cb-buffer (p-buffer)
  "Default implemention for `ac-candidate-function'."
  (save-excursion
    (set-buffer p-buffer)
    (if (> (length ac-prefix) 0)
	(let ((i 0)
	      candidate
	      candidates
	      (regexp (concat "\\b" (regexp-quote ac-prefix) "\\(\\s_\\|\\sw\\)*\\b")))
	  (save-excursion
	    ;; search backward
	    (goto-char (point-min))
	    (while (and (< i ac-limit)
			(re-search-forward regexp nil t))
	      (setq candidate (match-string-no-properties 0))
	      (unless (member candidate candidates)
		(push candidate candidates)
		(setq i (1+ i))))
          (nreverse candidates)))) ))


(provide 'auto-complete-magik)
