;; -------------------------------------------------------------------
;; Class Browser Styles
;; ====================
;;
;; To change color just redefine the faces in your _emacs file
;;
;; -------------------------------------------------------------------

(defface cb-font-lock-class-face          '((t (:foreground "red"))) "cb face" )
(defface cb-font-lock-comment-face        '((t (:foreground "light sea green" :height 0.8))) "cb face" )
(defface cb-font-lock-method-face         '((t (:background "yellow" :foreground "blue4"
							    :box (:line-width 1 :color "grey100" :style pressed-button)))) "cb face" )
(defface cb-font-lock-parameter-face      '((t (:foreground "deep pink"))) "cb face" )
(defface cb-font-lock-optional-face       '((t (:foreground "blue"))) "cb face" )
(defface cb-font-lock-user-method-face    '((t (:background "seagreen2" :foreground "navy"
							    :box (:line-width 1 :color "grey100" :style pressed-button)
							    :height 3))) "cb face" )

(defvar cb-font-lock-comment-face        'cb-font-lock-comment-face)
(defvar cb-font-lock-method-face         'cb-font-lock-method-face)
(defvar cb-font-lock-parameter-face      'cb-font-lock-parameter-face)
(defvar cb-font-lock-optional-face       'cb-font-lock-optional-face)
(defvar cb-font-lock-class-face          'cb-font-lock-class-face)
(defvar cb-font-lock-user-method-face    'cb-font-lock-user-method-face)

(defvar cb-font-lock-keywords
  '(
    ("  IN  [a-zA-Z0-9!_?<>:]+"     . cb-font-lock-class-face)
    ("  IN  "                       0 nil t)
    ("^[]a-zA-Z0-9!_?^<,>()\[]+"    . cb-font-lock-method-face)
    ("^user![]a-zA-Z0-9!_?^<,>()\[]+" 0 cb-font-lock-user-method-face t)
    ("^ .*$"                        . cb-font-lock-parameter-face)
    (" OPT .*$"                     0 cb-font-lock-optional-face t)
    ("#.*$"                         0 cb-font-lock-comment-face t)
    )
  "Class Browser")


(defun cb2-get-window (mode)
  "Set up a window for \"*cb2*\" and return nil if \"*cb2*\" already had a
window and was in the right mode.  (If it didn't have a window, the
buffer is re-filled from the cb global variables).  We also save
some state for a clean exit."
  (let ((cb2 (cb2-buffer)) ;actually always called from *cb2* buffer.
    (buf (cb-buffer)))
    ;; LIG : Customisation for Emacs Code Browser (ECB version 2.23)
    (if (get-buffer cb2)
    (display-buffer cb2))
    ;; -------------------------------------------------------------
    (if (get-buffer-window cb2)
    (progn
      (select-window (get-buffer-window cb2))
      (not (eq cb2-mode mode)))

    ;; else the window doesn't exist.
    (setq cb2-was-one-window (one-window-p t))
    (setq cb2-direct-p (not (get-buffer-window buf)))
    (let ((cb-win (get-buffer-window buf)))
      (cond
       ((not cb-win)
        ;; I'm not sure what to do here!  For now we'll just do the
        ;; same sort of start up as for "*cb*", and just note that the user
        ;; got into "*cb2*" without going via "*cb*".
        (setq cb-was-one-window (one-window-p t))
        ;;;;; doesn't appear to be used ;;;;;
        ;;(setq cb2-was-started-from-top-half
        ;;      (zerop (second (window-edges (selected-window)))))
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        (switch-to-buffer-other-window cb2))

       ((not (one-window-p t))
        ;; if there's 3 windows maybe we should try and zap the least wanted
        ;; window.  For now we just zap the next in rotation.
        (let
            ((cb2-win (next-window cb-win 1)))
          (set-window-buffer cb2-win cb2)
          (select-window cb2-win)))

       ;; Now the 2 cases when "*cb*" is occupying the whole screen.
       ;; The aim is to leave "*cb*" in the half of the screen that was
       ;; its original home before the user did a `C-x 1'.

       (cb-was-started-from-top-half
        ;; make "*cb2*" appear in the top half.
        (split-window-vertically)
        (switch-to-buffer cb2))

       (t
        (switch-to-buffer-other-window cb2))))
    t)))


;; LIG

(defvar f3-history '())

(define-key sw-f3-map "b"    'cb-jump-back)
(define-key sw-f3-map "j"    'cb-jump-to-source)

(defun cb-jump-back ()
  (interactive)
    (if (> (length f3-history) 0)
      (progn
        (switch-to-buffer (caar f3-history)) (goto-line (cdar f3-history))
        (setq f3-history (cdr f3-history))
        (message "F3j History size %d." (length f3-history))
        )
    (message "F3j History is empty"))
)


(defun cb-jump-to-source ()
  "Jump to the source for the method under the cursor."
  (interactive)
  (if (eq major-mode 'cb-mode)
      (cb-jump-to-source-from-cb)
    ;; LIG
    (progn
      (setq cb-temp-method-name (cb-curr-method-name))
      (setq f3-history (cons (cons (buffer-name) (what-line-number)) f3-history))
      (message "F3j History size %d." (length f3-history))
      (cb nil cb-temp-method-name "")
    )
  )
)

(defun what-line-number ()
  (interactive)
  (let ((opoint (point)) start)
    (save-excursion
      (save-restriction
        (goto-char (point-min))
        (widen)
        (beginning-of-line)
        (setq start (point))
        (goto-char opoint)
        (beginning-of-line)
        (if (/= start 1)
               (1+ (count-lines start (point)))
               (1+ (count-lines 1 (point))))))))

;; Custimisation by LIG to automatically select *cb**gis* buffer on F3-F3
;;
(defun cb (&optional gis method class)
  "Start or resume a Smallworld Class Browser.

With a prefix arg, ask user for GIS buffer to associate with.

Main top level entry to the cb.

Create the buffer and/or start the process if necessary.
Do a no-op if already in the cb."
  (interactive)
  (let (ver cb-file running-p buffer gis-proc visible-bufs bufs)
    (cond ((and (integerp current-prefix-arg) (> current-prefix-arg 0))
       (setq gis (sw-get-buffer-mode gis
                     'gis-mode
                     resources-gis-enter-buffer
                     (cond ((eq major-mode 'cb-mode) (cb-gis-buffer))
                           ((eq major-mode 'gis-mode) (buffer-name))
                           (t gis-buffer))
                     'gis-buffer-alist-prefix-function))
       (unless (get-buffer-process gis)
         (pop-to-buffer gis)
         (error resources-gis-no-process-error))
       (unless (get-buffer gis)
         (pop-to-buffer gis)
         (error resources-cb-none-running-error)))
      ((and (integerp current-prefix-arg) (< current-prefix-arg 0))
       (setq buffer (sw-get-buffer-mode nil
                        'cb-mode
                        resources-cb-enter-buffer
                        nil
                        'cb-buffer-alist-prefix-function
                        nil
                        'cb-filename))
       (unless (get-buffer buffer)
         (pop-to-buffer buffer)
         (error resources-cb-none-running-error)))
      (current-prefix-arg
       (setq cb-file (cb-set-filename)
         buffer (generate-new-buffer-name
             (concat "*cb*" "*" (or buffer (file-name-nondirectory cb-file)) "*"))
         gis    (cb-gis-buffer buffer)))
      ((eq major-mode 'cb-mode)
       (setq gis (cb-gis-buffer)))
      ((eq major-mode 'gis-mode)
       (setq gis (buffer-name)))
      ((and ;List of *visible* cb-mode *and* gis-mode buffers.
        (setq bufs
          (delete nil
              (mapcar (function (lambda (b) (if (cdr b) b)))
                  (setq visible-bufs
                    (sw-buffer-visible-list '(cb-mode gis-mode))))))
        ;;restrict list to those whose cdr is t.
        (setq buffer
          (if (= (length bufs) 1)
              (caar bufs)
            (completing-read
             (concat resources-cb-or-gis-enter-buffer " ")
             visible-bufs 'cdr t)))
        (not (equal buffer "")))
       (if (equal (substring buffer 0 4) "*cb*")
           nil ;;Selected a CB buffer
         (setq gis buffer
           buffer (concat "*cb*" buffer))))
      ((and
        visible-bufs
        (setq buffer
          (cond ((= (length visible-bufs) 1)
             (caar visible-bufs))
            ((and (= (length visible-bufs) 2)
                  (or (equal (substring (caar visible-bufs) 0 4) "*cb*")
		      (equal (substring (caadr visible-bufs) 0 4) "*cb*") ))
	     (caar visible-bufs))
            (t (completing-read
             (concat resources-cb-or-gis-enter-buffer " ")
             visible-bufs nil t))
            );; cond by LIG
          );;buffer
        (not (equal buffer "")))
       (select-frame-set-input-focus
        (window-frame (get-buffer-window buffer 'visible)))
       (if (equal (substring buffer 0 4) "*cb*")
           nil ;;Selected a CB buffer
         (setq gis buffer
           buffer (concat "*cb*"  buffer))))
      ((setq buffer (sw-get-buffer-mode nil
                        'cb-mode
                        resources-cb-enter-buffer
                        (let ((cb (concat "*cb*" gis-buffer)))
                        (if (get-buffer cb) cb))))
       t)
      ((and gis-buffer (get-buffer gis-buffer) (get-buffer-process gis-buffer))
       (setq gis gis-buffer))
      (t
       (setq cb-file (cb-set-filename)
         buffer (generate-new-buffer-name
             (concat "*cb*" "*" (file-name-nondirectory cb-file) "*"))
         gis    (cb-gis-buffer buffer))))

    (setq buffer   (or buffer (concat "*cb*" gis))
      gis-proc (and gis (get-buffer-process gis)))

    (cond ((cb-is-running buffer)
       (setq running-p t))
      ((and cb-dynamic gis-proc)
       (setq buffer (get-buffer-create buffer)))
      (t
       (setq gis-proc nil)))
    (pop-to-buffer buffer)
    (cb-mode)

    (if running-p
    (progn
      (if (cb-set-method-and-class method class)
          (cb-send-modeline-and-pr)
        (cb-redraw-modeline))
      (cb-set-windows))

      (setq ver (cb-method-finder-version)
        cb-quote-file-name   (string< "5.2.0" ver)
        cb-mf-extended-flags (string< "6.0.0" ver)
        cb-n-methods-str "0"
        cb-topic-pos 1)

      ;;Ensure buffers are empty
      (cb-set-method-str "")
      (cb-set-class-str "")

      (let ((buffer-read-only nil))
    (erase-buffer)
    (insert resources-cb-header))
      (goto-char (point-min))
      (cb-redraw-modeline)
      (let* ((process-environment (copy-list (save-excursion
                           (and gis (get-buffer gis) (set-buffer gis))
                           (or gis-process-environment
                           process-environment))))
         (gis-exec-path (copy-list (save-excursion
                     (and gis (get-buffer gis) (set-buffer gis))
                     (or gis-exec-path exec-path))))
         (acp-path (getenv "SW_ACP_PATH"))
         (exec-path
          (if acp-path
          (append (parse-colon-path acp-path) gis-exec-path)
        gis-exec-path)))
    (if gis-proc
        ;; then ask Magik to start a method_finder.  Magik will
        ;; tell us if it succeeds in starting a new method_finder.
        (let (cb-mf-socket-synchronised)
          (process-send-string gis-proc
                   "method_finder.send_socket_to_emacs()\n$\n")
          (while (not cb-mf-socket-synchronised)
        (sleep-for 0.01)))

      ;; otherwise start our own method_finder.
      (setq cb-filename cb-file)
      (cb-start-process "method_finder"
                "-e"
                ;; we give a socket-name or pipe-name
                ;; even though no-one is going to connect
                ;; to the method_finder.  This is because
                ;; the method_finder no longer has a single-user mode.
                (if (running-under-nt)
                (concat "\\\\.\\pipe\\method_finder\\time"
                    (number-to-string (first (current-time)))
                    "."
                    (number-to-string (second (current-time)))
                    "\\pointmax"
                    (number-to-string (point-max)))
                  (concat "/tmp/emacs_mf_sock" (number-to-string (emacs-pid)))))
      (cb-send-load cb-filename)))
      (setq cb-pending-message t)
      (message resources-cb-loading)

      ;; Update cb-buffer-alist using negative numbers if loading from a file,
      ;; positive numbers are used by gis-buffer-alist for loading from GIS
      (if (and cb-filename
           (not (rassoc (buffer-name) cb-buffer-alist)))
      (let ((n -1))
        (while (cdr (assq n cb-buffer-alist))
          (setq n (1- n)))
        (if (assq n cb-buffer-alist)
        (setcdr (assq n cb-buffer-alist) (buffer-name))
          (add-to-list 'cb-buffer-alist (cons n (buffer-name))))
        (assq n cb-buffer-alist)))

      (if (cb-set-method-and-class method class)
      (if cb-process
          (cb-send-modeline-and-pr)
        (error resources-cb-not-running-error (current-buffer)))
    (cb-redraw-modeline)
    (cb-set-windows)))))


(provide 'custom-cb)
