;; -------------------------------------------------------------------
;; GIS Buffer Styles
;; =================


;; -------------------------------------------------------------------
;; LIG - modified the original Smallworld code to work the 'old fashioned' way
;; - switch current buffer to *gis* - this is bound to <F2-g>
;; -------------------------------------------------------------------
(defun lig-gis (&optional buffer command)
  "Run a Gis process in a buffer in `gis-mode'.

The command is typically \"sw_magik_win32\" or \"sw_magik_motif\", but
can be any interactive program such as \"csh\".

The program that is offered as a default is stored in the variable,
`gis-command', which you can customise.  e.g.

\(setq gis-command
  \"[$HOME] sw_magik_win32 -Mextdir %TEMP% -image $SMALLWORLD_GIS/images/gis.msf\"
\)
The command automatically expands environment variables using
Windows %% and Unix $ and ${} nomenclature.

You can setup a list of standard commands by setting the
default value of `gis-command-history'.

Prefix argument controls:
With a numeric prefix arg, switch to the Gis process of that number
where the number indicates the order it was started. The
SW->Gis Processes indicates which numbers are in use. If a Gis process
buffer is killed, its number is reused when a new Gis process is started.

With a non-numeric prefix arg, ask user for buffer name to use for
GIS.  This will default to a unique currently unused name based upon
the current value of `gis-buffer-default-name'.

If there is already a Gis process running in a visible window or
frame, just switch to that buffer, or prompt if more than one.  If
there is not, prompt for a command to run, and then run it."

  (interactive)
  (if command (setq gis-command command))
  (let (dir
        cmd
        args
    ;;read-string's history arg does not work with buffer-local variables
    ;;history also always has something see Package Registration at end
        (command-history gis-command-history)
        alias-beg
        alias-expansion
    (alias-buffer "*temp gis alias buffer*")
        (keepgoing t)
    (gis-start-process-pre-hook gis-start-process-pre-hook)
    (buffer (sw-get-buffer-mode (cond (buffer buffer)
                      ((eq major-mode 'gis-mode) (buffer-name))
                      (t nil))
                    'gis-mode
                    resources-gis-enter-buffer
                    (or gis-buffer gis-buffer-default-name)
                    'gis-buffer-alist-prefix-function
                    (generate-new-buffer-name gis-buffer-default-name)))
        (rev-1920-regexp " +\\[rev\\(19\\|20\\)\\] +")
        (alias-subst-regexp "\\\\!\\(\\\\\\)?\\*"))
    (if (and (get-buffer-process buffer)
             (eq (process-status (get-buffer-process buffer)) 'run))
        (progn
      ;;LIG Replace this with old way - switch current buffer to 'gis' buffer
      (switch-to-buffer buffer)
      ;(pop-to-buffer buffer)
      (goto-char (point-max)))

      ;; Else start a fresh gis:
      ;; We keep going round expanding aliases until there is no alias expansion.
      ;; Each time round the user can edit the expanded alias.
      ;; We also silently remove any strings of the form [rev20] or [rev19].

      (save-excursion
        (set-buffer (get-buffer-create alias-buffer))
        (erase-buffer)
        (if (and (equal (getenv "SHELL") "/bin/csh")
                 (file-readable-p "~/.alias"))
            (insert-file-contents "~/.alias"))

        (while keepgoing
      (setq keepgoing nil)
      (setq gis-command (sub gis-command rev-1920-regexp " "))
      (or (eq (string-match "\\[" gis-command) 0)
          (setq gis-command (concat "[" default-directory "] " gis-command)))
      (or command
          (setq gis-command
            (read-string (concat resources-gis-command-prompt " ")
                 (car command-history)
                 'command-history)))
      (if (string-match rev-1920-regexp gis-command)
          (progn
        (setq keepgoing t)
        (setq gis-command (sub gis-command rev-1920-regexp " "))))
      (or (eq (string-match "\\[" gis-command) 0)
          (setq gis-command (concat "[" default-directory "] " gis-command)))
      (string-match "\\[\\([^\]]*\\)\\] *\\([^ ]*\\) *\\(.*\\)" gis-command)
      (setq dir  (substring gis-command (match-beginning 1) (match-end 1)))
      (setq cmd  (substring gis-command (match-beginning 2) (match-end 2)))
      (setq args (substring gis-command (match-beginning 3) (match-end 3)))

      (goto-char (point-min))
      (if (re-search-forward (concat "^alias[ \t]+" (regexp-quote cmd) "[ \t]+") nil t)
          (progn
        (setq keepgoing t)
        (setq alias-beg (match-end 0))
        (goto-char alias-beg)
        (if (looking-at "['\"]")
            (progn
              (incf alias-beg)
              (end-of-line)
              (re-search-backward "['\"]"))
          (end-of-line))
        (setq alias-expansion (buffer-substring alias-beg (point)))
        (or (string-match alias-subst-regexp alias-expansion)
            (setq alias-expansion (concat alias-expansion " \\!*")))
        (setq alias-expansion (sub alias-expansion alias-subst-regexp args))
        (setq gis-command (concat "[" dir "] " alias-expansion)))))

        (kill-buffer alias-buffer))

      (pop-to-buffer (get-buffer-create buffer))
      (gis-mode)
      (goto-char (point-max))
      (insert "\n" (current-time-string) "\n")
      (setq default-directory (expand-file-name
                   (file-name-as-directory
                (substitute-in-file-name dir)))
        gis-current-command (copy-sequence gis-command)
        gis-command-history (cons gis-current-command
                      (delete gis-current-command gis-command-history)))
      (setq-default gis-command-history (cons gis-current-command
                      (delete gis-current-command gis-command-history)))
      (or (file-directory-p default-directory)
      (error resources-sw-no-directory-error default-directory))
      (add-hook 'gis-start-process-pre-hook
        (function (lambda () (insert gis-command ?\n ?\n)))
        t)
      (gis-start-process (gis-parse-gis-command (concat cmd " " args))))))


;; -------------------------------------------------------------
;; modified function from the core SW Emacs.
;; Compiling a code (F7/F2-b) switching to the *gis* buffer/frame
;; while working with multiple Frames - doesn't get the compiled
;; magik code in to the Class Browser.
;; -------------------------------------------------------------

(defun sw-get-buffer-mode (buffer mode prompt default &optional prefix-fn initial predicate)
  "Generalised function to return a suitable major MODE buffer to use.

Used for determining a suitable BUFFER using the following interface:
1. If Prefix arg is given and is integer,
   then use the buffer returned from the PREFIX-FN.
   The buffer list is filtered according to PREDICATE if given.
2. If Prefix arg is given and is not an integer,
   then PROMPT user with completing list of known buffers
   (optionally provide an INITIAL value).
   The buffer list is filtered according to PREDICATE if given.
3. If BUFFER is given use that.
4. Use the buffer displayed in the current frame,
   only PROMPT if more than one buffer in current frame is displayed
   and only list those in the current frame.
5. Use the buffer displayed in the some other frame,
   only PROMPT if more than one buffer in the other frames are displayed
   and only list those that are displayed in the other frames.
6. Use DEFAULT value.
"
  (let* ((prefix-fn (or prefix-fn
            '(lambda (arg mode predicate)
               (nth (1- arg)
                (reverse (sw-buffer-mode-list-sorted mode predicate))))))
     (prompt (concat prompt " "))
     (visible-bufs (sw-buffer-visible-list mode predicate))
     bufs
     (buffer (cond ((and (integerp current-prefix-arg)
                  (setq buffer (funcall prefix-fn current-prefix-arg mode predicate)))
             buffer)
            (current-prefix-arg
             (completing-read prompt
                      (mapcar '(lambda (b) (cons b b))
                          (sw-buffer-mode-list mode predicate))
                      nil nil
                      initial))
            (buffer buffer)
            ((and
              (setq bufs
                (delete nil
                    (mapcar (function (lambda (b) (if (cdr b) b))) visible-bufs)))
              ;;restrict list to those whose cdr is t.
              (setq buffer
                (if (= (length bufs) 1)
                    (caar bufs)
                  (completing-read prompt visible-bufs 'cdr t)))
              (not (equal buffer "")))
             buffer)
            ((and
              visible-bufs
              (setq buffer
                (if (= (length visible-bufs) 1)
                    (caar visible-bufs)
                  (completing-read prompt visible-bufs nil t)))
              (not (equal buffer "")))
             ;LIG (select-frame-set-input-focus
             ;LIG (window-frame (get-buffer-window buffer 'visible)))
             buffer)
            (t default))))
    buffer))

(provide 'custom-gis)
