

;; Cursor colours
;;****************************************************************
(setq cua-normal-cursor-color "brown")
(setq cua-overwrite-cursor-color "green1")
(setq cua-read-only-cursor-color "red")


(defcustom initial-frame-width 135
  "WIDTH of the Initial Frame"
  :type 'integer
  :group 'misc-custom)

(defcustom initial-frame-height 43
  "HEIGHT of the Initial Frame"
  :type 'integer
  :group 'misc-custom)

(defcustom  custom-background-color "wheat"
  "Emacs background color"
  :type 'string
  :group 'misc-custom)

(defcustom  custom-modeline-background-color "wheat3"
  "Emacs modeline background color"
  :type 'string
  :group 'misc-custom)

(defcustom  custom-foreground-color "black"
  "Emacs foreground color"
  :type 'string
  :group 'misc-custom)


;; Save all backups in a single directory.
;;****************************************************************
(setq backup-dir (getenv "TEMP"))

(defcustom  custom-backup-directory backup-dir
  "Directory for Emacs backup *.*~ files. Need to restart Emacs in order for this setting to take effect."
  :type 'directory
  :group 'misc-custom)

(setq backup-directory custom-backup-directory)


;;****************************************************************
;; Setup the colours, and initial position of my frames.
;;  Colour fonts, etc
;;  Turn on font-lock in all modes that support it
;;  Maximum colors
;;****************************************************************
;;
;;  Set the default size, properties of the frame.
;; ************************************************=
(cond (window-system
       (progn
     (setq default-frame-alist
           (append default-frame-alist
           '( (top . 1) (left . 1)
              (width . 120)
              (height . 40)
              (icon-name        . 'question)
              ;(font . "-*-Courier-r-*-*-*-*-*-*-*-*-fontset-startup" . "fontset-startup")
              (font . "-outline-Courier new-normal-normal-normal-mono-*-*-*-*-c-*-iso8859-1")
	      )
	   (list (cons 'background-color custom-background-color))
	   (list (cons 'foreground-color custom-foreground-color))
	   ))
     (set-face-background 'modeline custom-modeline-background-color)
     )))
;;
;;  Set the default frame size, and positioning. for the initial frame.
;; ******************************************************************=
(cond (window-system
       (progn
     (setq initial-frame-alist
           (append
        '( (top . 1) (left . 1)
           ;(font . "-*-Courier-r-*-*-*-*-*-*-*-*-fontset-startup" . "fontset-startup")
           (font . "-outline-Courier new-normal-normal-normal-mono-*-*-*-*-c-*-iso8859-1")
	   ;(font . "-*-Courier-normal-normal-normal-*-12-*-*-*-c-*-iso8859-1")
           )
	(list (cons 'background-color custom-background-color))
	(list (cons 'foreground-color custom-foreground-color))
        (list (cons 'width initial-frame-width))
        (list (cons 'height initial-frame-height))
         ))
     )))


;;****************************************************************

(require 'time-stamp)
(add-hook 'write-file-hooks 'time-stamp)

(defvar project-name nil
"If not nil, this is inserted in a review block")

(defvar programmer-tag (user-login-name)
  "identifier used in magik note insertion functions")

(defvar programmer-time-stamp-format "%02y/%02m/%02d"
  "Template for the string produced by programmer-note.  See documentation on
time-stamp-format to see the various format options")

(defcustom custom-method-comment 'method-comment
  "Custom Method Header/Comment, see ~\site-lisp\custom-lisp\custom-magik-info.el to modify"
  :type 'function
  :group 'misc-custom)

(defun time-stamp-string()
   (interactive)
   (time-stamp-string-preprocess programmer-time-stamp-format)
)

(defun insert-programmer-file-header-in-empty-buffer()
  ""
  (interactive)
  (if (not (equal (point-max) (point-min)))
      ()
      (insert-programmer-file-header)
      (not-modified)
    )
  )

(defun insert-programmer-note ()
  "insert the start of a programmer note at point."
  (interactive)
  (insert (programmer-note))
  )

(defun insert-programmer-modified-note ()
  "insert a programmer-modified-note at point."
  (interactive)
  (insert (programmer-modified-note))
  )


(defun insert-method-comment ()
  "inserts a programmer-comment tag under the last one found at the top of the method"
  (interactive)
  (beginning-of-line)
  (open-line 1)
  (insert (method-comment))
  (magik-indent-command)
  ;(recenter)
  )

;; ------------------------------
;; Custom key-mappings ...
;; ------------------------------
(global-set-key "\M-n" 'insert-programmer-note)
(global-set-key "\M-m" 'insert-programmer-modified-note)
(global-set-key "\M-c" 'insert-change-note)
(global-set-key "\M-h" 'insert-method-comment)
;(global-set-key "\M- " 'hippie-expand)

(defvar magik-f3-map (make-sparse-keymap)
  "Keymap for the F2 function key in Magik buffers")
(fset 'magik-f3-map magik-f3-map)

(define-key magik-mode-map [f3]    'magik-f3-map)
(define-key magik-f3-map "t" 'lig_magik-trace-curr-statement)
(define-key magik-f3-map "d"  'toggle-transmit-magik-debug-p)

;(global-set-key [f3 ?d]  'toggle-transmit-magik-debug-p)
;(global-set-key [f3 ?t]  'lig_magik-trace-curr-statement)

(global-set-key [C-delete] 'kill-word)

(global-set-key [(control tab)]    'tabbar-forward-tab)
(global-set-key [(control ?`)]     'tabbar-press-home)


;;******************** Setup Speedbar ********************************************=
(require 'speedbar)
(global-set-key [f11] 'speedbar-get-focus) ;; Define F11 to toggle between emacs and speedbar frames
(global-set-key [f12 f12] 'w32shell-explorer-here) ;Opens Windows Explorer in current dirrectory
(speedbar-add-supported-extension "magik") ;; .magik files are of interest
(speedbar-add-supported-extension "msg")   ;; .msg files are of interest
(speedbar-add-supported-extension "cfg")   ;; .cfg files are of interest
(speedbar-add-supported-extension "def")   ;; .def files are of interest
(speedbar-add-supported-extension "txt")   ;; .txt files are of interest
(speedbar-add-supported-extension "xml")   ;; .xml files are of interest
;; For some reason XML files does not handled by Speedbar

;; ********************** ECB key mappings **************************************=
;; ?????????? conficting with core Emacs buttons ??????????
(global-set-key [f12 ?b] 'ecb-minor-mode)
(global-set-key [f12 ?w] 'ecb-toggle-ecb-windows)
(global-set-key [f12 ?t] 'ecb-toggle-layout)

(global-set-key [f12 ?q] 'ecb-redraw-layout)
(global-set-key [f12 ?r] 'ecb-restore-window-sizes)
(global-set-key [f12 ?u] 'ecb-rebuild-methods-buffer)

(global-set-key [f12 ?c] 'ecb-toggle-compile-window)
(global-set-key [f12 ?h] 'ecb-toggle-compile-window-height)

(global-set-key [f12 ?-] 'ecb-cycle-maximized-ecb-buffers)
(global-set-key [f12 ?=] 'ecb-maximize-window-methods)

;; ************************** Bookmarks **********************************
(global-set-key [f5 ?m] 'bookmark-set)
(global-set-key [f5 ?j] 'edit-bookmarks)
(global-set-key [f5 ?e] 'edit-bookmarks)
(global-set-key [f5 ?d] 'bookmark-delete)
(global-set-key [f5 ?s] 'bookmark-write)
(global-set-key [f5 ?l] 'bookmark-load)

;; ************************** Visual Bookmarks **********************************
(global-set-key [C-f5]  'bm-toggle)
(global-set-key [f5 ?n] 'bm-next)
(global-set-key [f5 ?p] 'bm-previous)
(global-set-key [f5 ?a] 'bm-toggle-cycle-all-buffers)

;; ************************** Breadcrumbs ******************************************
(define-key esc-map " "   'bc-set)
(global-set-key [f5 ?b]   'bc-next)
(global-set-key [M-f5]    'bc-list)


;; ************************** MISC ******************************************
(define-key sw-f2-map "g"    'lig-gis)

(global-set-key [f12 ?e] 'send-file-to-external-editor)
(global-set-key [f12 ?m] 'magik-copy-method-to-buffer) ;;NOW SET TO F4-m

(global-set-key [f4 ?p] "print(!)")
(define-key esc-map "z" 'shell)

;; ************************** Outline mode ******************************************
(define-key magik-f3-map [up]   'hide-entry)
(define-key magik-f3-map [down] 'show-entry)

;(sw-define-key magik-mode-map [M-up]   'hide-entry)
;(sw-define-key magik-mode-map [M-down] 'show-entry)

;; ************************** Move Text ******************************************
;; Key bindings. (Eclipse style)
;(global-set-key [M-up] 'move-text-up)
;(global-set-key [M-down] 'move-text-down)

(global-set-key [C-M-up]      'move-text-up)
(global-set-key [C-M-down]    'move-text-down)



;; ************************** Hide/Show mode ******************************************
(setq hs-org/trigger-keys-block (list (kbd "C-.")))
(setq hs-org/trigger-keys-all   (list (kbd "C-,")))


(define-key global-map [f12 ?n]  'highlight-changes-next-change)
(define-key global-map [f12 ?p]  'highlight-changes-previous-change)
(define-key global-map "\C-x\C-q" 'lig-toggle-read-only)

;; **************************  NXML <-> SGML/XML mode switch **********************
(global-set-key [f12 ?x] 'switch-xml-mode)

(global-set-key "\C-x\C-b" 'bs-show)

(global-set-key (kbd "<M-S-up>")     'buf-move-up)
(global-set-key (kbd "<M-S-down>")   'buf-move-down)
(global-set-key (kbd "<M-S-left>")   'buf-move-left)
(global-set-key (kbd "<M-S-right>")  'buf-move-right)

(define-key global-map (kbd "M-b")     'other-window)
(define-key global-map (kbd "M-f")     'other-frame)
(define-key global-map (kbd "M-d")     'delete-other-windows)

(global-set-key "\M-/" 'dabbrev-expand-multiple)
(global-set-key "\C-ch" 'hs-org/minor-mode)

;; -------------------------------------------------------------
;; kill current buffer without confirmation
;; -------------------------------------------------------------
(global-set-key "\C-xk" 'kill-current-buffer)


(provide 'custom-vars)
