;; Copyright (C) 2004-2008 Igor G Lapitski
;;
;; Author: Igor G Lapitski

;; This file is not part of GNU Emacs.

;;; License
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License version 2 as
;; published by the Free Software Foundation.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
;;

(message "------ Loading Custom code......")
(message (emacs-uptime))



(defgroup misc-custom nil
  "LIG Custom Emacs User Setting"
  :group 'smallworld)

(defun lig-set-ecb-mode (symbol value)
  (custom-set-default symbol value)
  (if (eq value t)
      (lig-load-ecb))
)

(defcustom ecb-load nil
  "ECB load on Emacs start up?"
  :type '(choice (const :tag "Yes" t)
                 (const :tag "No" nil))
  :initialize 'custom-initialize-default
  :set 'lig-set-ecb-mode
  :group 'misc-custom)

(defun lig-set-pair-mode (symbol value)
  (custom-set-default symbol value)
  (if (eq value t)
    (setq skeleton-pair t)
    (setq skeleton-pair nil)
    )
)

(defcustom pair-mode nil
  "Pairing mode on Emacs start up?"
  :type '(choice (const :tag "ON" t)
                 (const :tag "OFF" nil))
  :initialize 'custom-initialize-default
  :set 'lig-set-pair-mode
  :group 'misc-custom)

(defcustom toggle-pabbrev-mode t
  "Predictive Abbreviation on Emacs start up?"
  :type '(choice (const :tag "ON" t)
                 (const :tag "OFF" nil))
  :group 'misc-custom)

(defcustom toggle-ac-auto-start nil
  "Non-nil means completion will be started automatically.
Positive integer means if a length of a word you entered is larger than the value,
completion will be started automatically."
  :type '(choice (const   :tag "Never" nil)
                 (integer :tag "Require"))
  :group 'misc-custom)


(defun lig-load-ecb ()
  (if ecb-load
    (progn
      (message "Loading ECB related code ....")
;; -------------------------------------------------------------
;; Installation of Collection of Emacs Development Environment
;; Tools (CEDET)
;; -------------------------------------------------------------
      (load-file (substitute-in-file-name "$EMACSROOT/site-lisp/cedet-1.0pre7/common/cedet.el"))
      (semantic-load-enable-minimum-features)

;; -------------------------------------------------------------
;; Installation of The Emacs Lisp Library
;; -------------------------------------------------------------
      (add-to-list 'load-path (substitute-in-file-name "$EMACSROOT/site-lisp/elib-1.0"))

;; -------------------------------------------------------------
;; Installation instructions for Emacs Code Browser (ECB)
;; version 2.40
;; -------------------------------------------------------------
      (add-to-list 'load-path (substitute-in-file-name "$EMACSROOT/site-lisp/ecb-2.40"))
      (require 'ecb)
      (message "ECB is ready")
      )
    (message "ECB code not loaded")
    )
);; lig-load-ecb

;; Load ECB at start up if needed
(lig-load-ecb)

;; Load custom styles and modifications
;;********************************************************************
;; check original font-locking for magik
(require 'custom-magik)
(require 'custom-cb)
(require 'custom-gis)
(require 'custom-vars)
(require 'msmeller)
(require 'custom-toolbar)
(require 'move-text)

;; -------------------------------------------------------------
;; Load some specific Magik Files Information: User Comments,
;; File Headings, etc..
;; -------------------------------------------------------------
(load "custom-magik-info.el")

;; -------------------------------------------------------------
;; That is where all User local customisations can be placed -
;; in your [USER_EMACS=]%USERPROFILE%\Emacs_local_settings.el
;; Next line of the code will load them in to the Emacs session
;; Load some USER local customization from %USER_EMACS%
;; (see gis_aliases for details)
;; -------------------------------------------------------------
(if (getenv "USER_EMACS")
    (progn (if (file-exists-p (getenv "USER_EMACS"))
                (load (getenv "USER_EMACS")) ) )
)
;; -------------------------------------------------------------
;; This lines need to be in your [set USER_EMACS=]%USERPROFILE%\Emacs_local_settings.el
;; -------------------------------------------------------------
;; #############################################################
;; ##  change this in the [Emacs_local_settings.el] to show your
;; ##  name
;; #############################################################
;;(setq programmer-tag "Igor G Lapitski, HYDE PARK")
;;(setq project-name "Project Name")

;; -------------------------------------------------------------
;; Installation instructions for Aspell
;; -------------------------------------------------------------
(setq-default ispell-program-name (substitute-in-file-name "$EMACSROOT/Aspell/bin/aspell.exe"))

;; -------------------------------------------------------------
;; Set Language environment for Magik to Latin-1
;; This language environment is a generic one for Latin-1 (ISO-8859-1)
;; character set which supports the following languages:
;; Danish, Dutch, English, Faeroese, Finnish, French, German, Icelandic,
;; Irish, Italian, Norwegian, Portuguese, Spanish, and Swedish.
;; -------------------------------------------------------------
(if (equal current-language-environment "English")
    (set-language-environment "Latin-1"))

;; -------------------------------------------------------------
;; More fine-grained minibuffer completion feedback scheme.
;; Prospective completions are concisely indicated within the
;; minibuffer itself, with each successive keystroke.
;; -------------------------------------------------------------
(require 'icomplete)
(icomplete-mode)

;; -------------------------------------------------------------
;; Stop Emacs from asking for "y-e-s", when a "y" will do.
;; -------------------------------------------------------------
(fset 'yes-or-no-p 'y-or-n-p)

;; -------------------------------------------------------------
;; Switch ON Auto Revert mode
;; -------------------------------------------------------------
(global-auto-revert-mode 1)

;; -------------------------------------------------------------
;; Switch ON Electric Magik Mode
;; -------------------------------------------------------------
(electric-magik-toggle)

;; -------------------------------------------------------------
;; If you edit XML using iso-8859-N encodings other than iso-8859-1 and
;; you are running Emacs 21.3 or later, then I recommend enabling
;; unify-8859-on-decoding-mode, by adding
;; -------------------------------------------------------------
(unify-8859-on-decoding-mode)

;; -------------------------------------------------------------
;; PSGML is a major mode for editing SGML and XML documents
;; -------------------------------------------------------------
(autoload 'sgml-mode "psgml" "Major mode to edit SGML files." t)
(autoload 'xml-mode "psgml" "Major mode to edit XML files." t)

;; -------------------------------------------------------------
;; Switch ON MSB Mode
;; -------------------------------------------------------------
(msb-mode)

;; -------------------------------------------------------------
;; Per default, the time and date goes into the mode line.
;; -------------------------------------------------------------
(remove-hook 'global-mode-string 'display-time-string)
(setq display-time-24hr-format t)
(setq display-time-day-and-date t)
(display-time)

;; -------------------------------------------------------------
;; This will show full File Path in the frame title
;; -------------------------------------------------------------
(setq frame-title-format '("GNU Emacs " emacs-version " [" (buffer-file-name " %f") "]  -= "display-time-string" -="))

;; -------------------------------------------------------------
;; Highlight matching parenthesises much more sophisticated
;; -------------------------------------------------------------
(setq blink-matching-paren-distance 500000) ;maximum distance to search for paren
(load "mic-paren")
(paren-activate)
(eval-if-gnu-emacs    ; as opposed to XEmacs.
(setq show-paren-face 'secondary-selection))
(show-paren-mode 1)

;; -------------------------------------------------------------
;; Edit all lines matching a given regexp
;; -------------------------------------------------------------
(autoload 'all "all" nil t)

;; -------------------------------------------------------------
;; Menu for selecting and displaying buffers, alternative for C-xC-b
;; -------------------------------------------------------------
(require 'bs)

;; -------------------------------------------------------------
;; Tabbar support
;; -------------------------------------------------------------
(require 'tabbar)
(tabbar-mode)

;; -------------------------------------------------------------
;; Additional menu for visiting files that were operated
;; on recently
;; -------------------------------------------------------------
(require 'recentf)

;; -------------------------------------------------------------
;; Line number mode for Emacs
;; -------------------------------------------------------------
(require 'setnu)

;; -------------------------------------------------------------
;; Save all backups in a single directory. The directory is defined
;; 'custom-vars.el' and it is set to %TEMP%
;; -------------------------------------------------------------
(require 'backups)
(move-backups t)

;; -------------------------------------------------------------
;; Minor mode to highlight current line in buffer.
;; -------------------------------------------------------------
(require 'highline)
(highline-mode t)

;; -------------------------------------------------------------
;; cmd-mode is automatically activated when you open a file whose
;; extention is "cmd" or "bat".
;; -------------------------------------------------------------
(autoload 'cmd-mode "cmd-mode" "CMD mode." t)
(setq auto-mode-alist (append '(("\\.\\(cmd\\|bat\\)$" . cmd-mode))
                                  auto-mode-alist))


;; -------------------------------------------------------------
;; Custom Functions
;; -------------------------------------------------------------
(defun kill-current-buffer ()
  "Kill the current buffer, without confirmation."
  (interactive)
  (kill-buffer (current-buffer)))

(defun lig-toggle-read-only (&optional verbose)
  "Change read-only status of current buffer"
  (interactive "P")
  (toggle-read-only))

(defun lig-outl-mode ()
    (interactive)
    (outline-minor-mode)
    (if outline-minor-mode
        (hide-body)))

(defun kill-entire-line (&optional arg)
  (interactive "P")
  (beginning-of-line)
  (kill-line (if arg arg 1)))

(defun indent-whole-file  (&optional arg)
  (interactive "P")
  ( save-excursion
    (indent-region (goto-char (point-min)) ( goto-char (point-max)) nil)))

(defun send-file-to-external-editor ()
  (interactive)
  (call-process (if (getenv "EXT_EDITOR")
        (getenv "EXT_EDITOR") "notepad.exe") nil 0 nil (buffer-file-name))
)

(defun lig-show-info ()
  (interactive)
  (let*
      ((info-file-name (expand-file-name (substitute-in-file-name "$EMACSROOT/site-lisp/custom-lisp/info.txt") (getenv "EMACSROOT") ))
       )
    (switch-to-buffer (get-buffer-create "*<Customisation Info>*"))
    (erase-buffer)
    (insert-file-contents info-file-name)
    (if (not buffer-read-only)
        (toggle-read-only)
        )
    (delete-other-windows)
    (goto-char (point-min))
    )
   )

(defun w32shell-explorer(dir)
  "Open Windows Explorer in directory DIR."
  (interactive "DStart in directory: ")
  (setq dir (expand-file-name dir))
  (w32-shell-execute nil dir))

(defun w32shell-explorer-here()
  "Open Windows Explorer in current directory."
  (interactive)
  (w32shell-explorer default-directory))

(defun lig-tabbar-buffer-list ()
  "Return the list of buffers to show in tabs.
Exclude buffers whose name starts with a space, when they are not
visiting a file."
  (delq t
        (mapcar #'(lambda (b)
                    (cond
                     ((buffer-file-name b) b)
                     ((char-equal ?\  (aref (buffer-name b) 0)))
             ((equal "*scratch*" (buffer-name b)))
             ((equal "*<Customisation Info>*" (buffer-name b)))
             ((equal "*Messages*" (buffer-name b)))
             ((equal "*history-gis*" (buffer-name b)))
             ((equal "*filter**gis*" (buffer-name b)))
             ((equal "*desktop*" (buffer-name b)))
             ((equal "*etags tmp*" (buffer-name b)))
             ((equal "*buffer-selection*" (buffer-name b)))
                     (b)))
                (buffer-list))))

(setq ecb-compilation-buffer-names-internal `(("*cb**gis*" . t)
                          ("*Buffer List*" . t)
                          ("*Bookmark List*" . t)
                          ("*history-gis*" . t)
                          ("*scratch*" . t)
                          ("*All*" . t)
                          ("*completion-save-buffer*" . t)
                          ("*buffer-selection*" . t)))

(add-hook 'ecb-activate-hook
          (lambda ()
                (ecb-toggle-compile-window -1)
		))

(defun pair-mode ()
  "toggle skeleton pair mode ON/OFF"
  (interactive)
  (if (eq skeleton-pair t)
    (progn (setq skeleton-pair nil)
	   (setq pair-mode nil)
	   (message "Pairing OFF"))
    (progn (setq skeleton-pair t)
	   (setq pair-mode t)
      (message "Pairing ON"))
 ))

(defun lock-mode-on ()
  "Lock current buffer"
  (interactive)
  (if (eq dedicated-mode nil)
      (dedicated-mode)
 ))

(defun lock-mode-off ()
  "UnLock current buffer"
  (interactive)
  (if (eq dedicated-mode t)
      (dedicated-mode)
 ))

(defun hs-mode-on ()
  "Hide/show mode ON"
  (interactive)
  (if (eq hs-org/minor-mode nil)
      (hs-org/minor-mode)
 ))

(defun hs-mode-off ()
  "Hide/show mode OFF"
  (interactive)
  (if (eq hs-org/minor-mode t)
      (hs-org/minor-mode)
 ))
;; ---------------------------------------------
;; NXML <-> SGML/XML mode switch
;; ---------------------------------------------
(defun switch-xml-mode ()
  "switch between NXLM and SGML/XML mode"
  (interactive)
  (if (eq major-mode 'nxml-mode)
      (xml-mode) (nxml-mode))
  )

;; -------------------------------------------------------------
;; Noticeable-Minibuffer-Prompts, copied and modified from Emacswin32
;; -------------------------------------------------------------
(define-minor-mode noticeable-minibuffer-prompts-mode
  "Use a more noticeable minibuffer prompt face when this mode is on.
Turning this mode on changes the face used for minibuffer prompts to
`minibuffer-noticeable-prompt'.  Turning it off changes it to
`minibuffer-prompt'."
 :init-value t
 :global t
 :group 'basic-faces
 (let ((face (member 'face minibuffer-prompt-properties)))
   (if noticeable-minibuffer-prompts-mode
       (setcar (cdr face) 'minibuffer-noticeable-prompt)
     (setcar (cdr face) 'minibuffer-prompt))))

(defface minibuffer-prompt
  '((((background dark)) :foreground "cyan")
    ;; Don't use blue because many users of the MS-DOS port customize
    ;; their foreground color to be blue.
    (((type pc)) :foreground "magenta")
    (t :foreground "medium blue"))
  "Face for minibuffer prompts.
By default, Emacs automatically adds this face to the value of
`minibuffer-prompt-properties', which is a list of text properties
used to display the prompt text.

See also `minibuffer-noticeable-prompt'."
  :version "22.1"
  :group 'basic-faces)

;; Modelled after secondary-selection face.
;; Suggested on Emacs Devel as usual.
(defface minibuffer-noticeable-prompt
  '((((class color) (min-colors 88) (background light))
     :background "gold")
    (((class color) (min-colors 88) (background dark))
     :background "SkyBlue4")
    (((class color) (min-colors 16) (background light))
     :background "yellow")
    (((class color) (min-colors 16) (background dark))
     :background "SkyBlue4")
    (((class color) (min-colors 8))
     :background "cyan" :foreground "black")
    (t :inverse-video t))
  "More noticeable face for minibuffer prompts.
You can switch between using this face and the face
`minibuffer-prompt' for the minibuffer prompt with the function
`noticeable-minibuffer-prompts-mode'."
  :version "22.1"
  :group 'basic-faces)

 (setq minibuffer-prompt-properties
      (append minibuffer-prompt-properties (list 'face 'minibuffer-noticeable-prompt)))
(put 'minibuffer-prompt-properties 'standard-value (list (list 'quote minibuffer-prompt-properties)))
(noticeable-minibuffer-prompts-mode t)



;; -------------------------------------------------------------
;; Save/Restore sessions
;; -------------------------------------------------------------
(defun dtop-read ()
  "Read the Desktop file and the files it specifies.
This is a no-op when Emacs is running in batch mode."
  (interactive)
  (dtop-get-file)
  (if noninteractive
      nil
    (if desktop-dirname
    (progn
        (let ((desktop-first-buffer nil)
             (desktop-buffer-ok-count 0)
             (desktop-buffer-fail-count 0))
       ;; Evaluate desktop buffer.
        (load (expand-file-name desktop-base-file-name desktop-dirname) t t t)
        ;; `desktop-create-buffer' puts buffers at end of the buffer list.
        ;; We want buffers existing prior to evaluating the desktop (and not reused)
        ;; to be placed at the end of the buffer list, so we move them here.
        ;(mapcar 'bury-buffer
        ;        (nreverse (cdr (memq desktop-first-buffer (nreverse (buffer-list))))))
        (switch-to-buffer (car (buffer-list)))
        (run-hooks 'desktop-delay-hook)
        (setq desktop-delay-hook nil)
        (run-hooks 'desktop-after-read-hook)
        (message "Desktop buffer restored.")
    (if ecb-minor-mode (ecb-add-all-buffers-to-history))
        t)
      );progn
      )
    )
)
;; ----------------------------------------------------------------------------
(defun dtop-get-file ()
  (interactive)
  (let ((buffer "") (file-fqn "") (lines-in-page 0) (dir-path ""))
    (setq buffer (generate-new-buffer "dtop"))
    (if (call-process dlgopen-executable-path nil buffer)
        (save-excursion
          (set-buffer buffer) (goto-line 1)
          (setq dir-path (get-current-line))
      (setq desktop-basefilename (file-name-nondirectory dir-path))
      (setq desktop-dirname (file-name-directory dir-path))
      (setq dirname (file-name-directory dir-path))
          ))
    (kill-buffer buffer))
    ;; Enable Desktop mode
    (desktop-save-mode 1)
  )
;; ----------------------------------------------------------------------------
(defun dtop-save ()
  "Save the Desktop file.  Parameter DIRNAME specifies where to save desktop."
  (interactive)
  (dtop-get-file)
  (if noninteractive
      nil
    (if desktop-dirname
    (progn
    (run-hooks 'desktop-save-hook)
  (setq dirname (file-name-as-directory (expand-file-name dirname)))
  (save-excursion
    (let ((filename (expand-file-name
             (concat dirname desktop-basefilename)))
    ;desktop-base-file-name dirname))
          (info
            (mapcar
              #'(lambda (b)
                  (set-buffer b)
                  (list
                    (desktop-file-name (buffer-file-name) dirname)
                    (buffer-name)
                    major-mode
                    ;; minor modes
                    (let (ret)
                      (mapc
                        #'(lambda (minor-mode)
                          (and
                            (boundp minor-mode)
                            (symbol-value minor-mode)
                            (let ((special (assq minor-mode desktop-minor-mode-table)))
                              (when (or special (functionp minor-mode))
                                (setq ret
                                  (cons
                                    (if special (cadr special) minor-mode)
                                    ret))))))
                        (mapcar #'car minor-mode-alist))
                      ret)
                    (point)
                    (list (mark t) mark-active)
                    buffer-read-only
                    ;; Auxiliary information
                    (when (functionp desktop-save-buffer)
                      (funcall desktop-save-buffer dirname))
                    (let ((locals desktop-locals-to-save)
                          (loclist (buffer-local-variables))
                          (ll))
                      (while locals
                        (let ((here (assq (car locals) loclist)))
                          (if here
                            (setq ll (cons here ll))
                            (when (member (car locals) loclist)
                              (setq ll (cons (car locals) ll)))))
                        (setq locals (cdr locals)))
                      ll)))
              (buffer-list)))
          (buf (get-buffer-create "*desktop*")))
      (set-buffer buf)
      (erase-buffer)

      (insert
        ";; -*- coding: emacs-mule; -*-\n"
        desktop-header
        ";; Created " (current-time-string) "\n"
        ";; Desktop file format version " desktop-file-version "\n"
        ";; Emacs version " emacs-version "\n\n"
        ";; Global section:\n")
      (mapcar (function desktop-outvar) desktop-globals-to-save)
      (if (memq 'kill-ring desktop-globals-to-save)
        (insert
          "(setq kill-ring-yank-pointer (nthcdr "
          (int-to-string (- (length kill-ring) (length kill-ring-yank-pointer)))
          " kill-ring))\n"))

      (insert "\n;; Buffer section -- buffers listed in same order as in buffer list:\n")
      (mapcar #'(lambda (l)
                  (if (apply 'desktop-save-buffer-p l)
                      (progn
                        (insert "(desktop-create-buffer " desktop-file-version)
                        (mapcar #'(lambda (e)
                                    (insert "\n  " (desktop-value-to-string e)))
                                l)
                        (insert ")\n\n"))))
              info)
      (setq default-directory dirname)
      (when (file-exists-p filename) (delete-file filename))
      (let ((coding-system-for-write 'emacs-mule))
        (write-region (point-min) (point-max) filename nil 'nomessage))))
  (setq desktop-dirname dirname)
))))

(setq desktop-missing-file-warning nil)

;;-------------- Windows Print -----------------------
(require 'htmlize-view)
(htmlize-view-add-to-files-menu)
(require 'w32-print)
(w32-print-add-to-files-menu)

;; -------------------------------------------------------------
;; swap buffers without typing C-x b
;; -------------------------------------------------------------
(require 'buffer-move)

;; -------------------------------------------------------------
;; Lock buffer so Emacs will not select files into that
;; window
;; -------------------------------------------------------------
(require 'dedicated)

;; -------------------------------------------------------------
;;  The Breadcrumb package allows you to set a series of breadcrumb bookmarks
;;  in Emacs and later jump to them quickly.
;; -------------------------------------------------------------
(require 'breadcrumb)

;; -------------------------------------------------------------
;;  The Visual Bookmarks
;; -------------------------------------------------------------
(require 'bm)

;; -------------------------------------------------------------
;;  With the hi-lock commands text matching interactively entered
;;  regexp's can be highlighted.
;; -------------------------------------------------------------
(global-hi-lock-mode 1)

;; -------------------------------------------------------------
;; enable skeleton-pair insert globally based on pair-mode value
;; -------------------------------------------------------------
(setq skeleton-pair pair-mode)    ;; set to <t> to use pairing function
(setq skeleton-pair-on-word nil)  ;; don't apply skeleton trick in front of a word.
(global-set-key (kbd "(")  'skeleton-pair-insert-maybe)
(global-set-key (kbd "[")  'skeleton-pair-insert-maybe)
(global-set-key (kbd "{")  'skeleton-pair-insert-maybe)
(global-set-key (kbd "\"") 'skeleton-pair-insert-maybe)

;;-------------------- yasnippet 0.5.1----------------------
 (add-to-list 'load-path
                 (substitute-in-file-name "$EMACSROOT/site-lisp/custom-lisp/yasnippet-0.5.10"))
 (require 'yasnippet) ;; not yasnippet-bundle

 (mapc '(lambda (x)
         (add-to-list 'yas/extra-mode-hooks
                      x))
      '(magik-mode-hook gis-mode-hook shell-mode-hook))

  (add-hook 'magik-mode-hook
             '(lambda ()
                (setq yas/buffer-local-condition
                      '(if (magik-in-comment-string)
                           '(require-snippet-condition . force-in-comment)
                         t))))

  (yas/initialize)
  (yas/load-directory (substitute-in-file-name "$EMACSROOT/site-lisp/custom-lisp/yasnippet-0.5.10/snippets"))

  (setq yas/window-system-popup-function
         'yas/x-popup-menu-for-template)

;;-------------------- yasnippet 0.6.1----------------------

;;  (add-to-list 'load-path
;;                     (substitute-in-file-name "$EMACSROOT/site-lisp/custom-lisp/yasnippet-0.6.1c"))

;;  (require 'yasnippet) ;; not yasnippet-bundle

;; (add-hook 'magik-mode-hook
;;           '(lambda ()
;;              (setq yas/buffer-local-condition
;;                    '(if (magik-in-comment-string)
;;                         '(require-snippet-condition . force-in-comment)
;;                       t))))

;;  (yas/initialize)
;;  (yas/load-directory (substitute-in-file-name "$EMACSROOT/site-lisp/custom-lisp/yasnippet-0.6.1c/snippets"))

;;-------------------- yasnippet END ----------------------

;; -------------------------------------------------------------
;; Dynamic abbrev for multiple selection.
;; -------------------------------------------------------------
(require 'dabbrev-expand-multiple)
;; put highlight to first expanded string.
(setq dabbrev-expand-multiple-highlight-face 'ac-candidate-face)

;; -------------------------------------------------------------
;; Predictive abbreviation expansion
;; -------------------------------------------------------------
;; REM this to use auto-complete (see below)
(require 'pabbrev)
(if toggle-pabbrev-mode
    (progn
      (add-hook 'magik-mode-hook 'pabbrev-global-mode)
      (add-hook 'gis-mode-hook   'pabbrev-global-mode)
      ))
(setq pabbrev-idle-timer-verbose nil)
;; LIG: see more below after 'auto completion' section


;; -------------------------------------------------------------
;; Inline Auto Completion
;; -------------------------------------------------------------
(add-to-list 'load-path (substitute-in-file-name "$EMACSROOT/site-lisp/custom-lisp/auto-complete-1.0"))
(require 'auto-complete-init)

;;----------------------------------------------
;; Predictive abbreviation expansion Customisation
;; This will use a pop-up list via auto-competion mode.
;; Make sure this is executed after pabbrev is loaded.
;;----------------------------------------------
(defun pabbrevx-ac-on-pre-command ()
  (if (or (eq this-command 'self-insert-command)
      (and (not (ac-trigger-command-p this-command))
	       (or (not (symbolp this-command))
		   (not (string-match "^ac-" (symbol-name this-command))))))
      (progn
	(remove-hook 'post-command-hook 'pabbrevx-ac-on-post-command t)
	(remove-hook 'pre-command-hook 'pabbrevx-ac-on-pre-command t)
	(ac-abort))))

(defun pabbrevx-ac-on-post-command ()
  (if (and (not isearch-mode)
       (ac-trigger-command-p this-command))
      (pabbrevx-ac-start)))

(defun pabbrevx-ac-start ()
  (let ((candidates (mapcar 'car pabbrev-expansion-suggestions)))
    (add-hook 'pre-command-hook 'pabbrevx-ac-on-pre-command nil t)
    (add-hook 'post-command-hook 'pabbrevx-ac-on-post-command nil t)
    (let* ((n (dropdown-list candidates)))
      (if n
	  (pabbrev-suggestions-insert-lig (nth n candidates))
	nil))))

(defun pabbrev-suggestions-insert-lig (insertion)
  "Actually insert the suggestion."
  (let ((point))
    (save-excursion
      (let ((bounds (pabbrev-bounds-of-thing-at-point)))
	(progn
	  (delete-region (car bounds) (cdr bounds))
	  (insert insertion)
	  (setq point (point)))))
    (if point
	(goto-char point))))

(defun pabbrevx-suggestions-goto-buffer (suggestions)
  (pabbrevx-ac-start))
(fset 'pabbrev-suggestions-goto-buffer 'pabbrevx-suggestions-goto-buffer)

(defvar toggle-ac-auto-start-mode
  (if (eq ac-auto-start nil)
      nil
    t)
)

(defun toggle-ac-auto-start-mode ()
  "toggle auto complete mode OFF or start after specified number of characters"
  (interactive)
  (if (eq toggle-ac-auto-start nil)
      (progn (setq ac-auto-start nil)
	     (message "Customise 'Toggle AC Auto Start' variable. Use <Alt-[> to activate AC menu, <Alt-]> to close it."))
    (progn (if (eq toggle-ac-auto-start-mode t)
	       (progn (setq ac-auto-start nil)
		      (setq toggle-ac-auto-start-mode nil)
		      (message "Use <Alt-[> to activate AC menu, <Alt-]> to close it."))
	     (progn (setq ac-auto-start toggle-ac-auto-start)
		    (setq toggle-ac-auto-start-mode t)
		    (if (eq pabbrev-mode t)
			(progn (global-pabbrev-mode)
			       (setq toggle-pabbrev-mode nil)))
		    (message "Completion will start automatically"))
))))

(defun toggle-pabbrev-mode()
  "toggle pabbrev mode ON/OFF"
  (interactive)
  (if (eq pabbrev-mode t)
    (progn (global-pabbrev-mode)
	   (setq toggle-pabbrev-mode nil)
	   (message "Global Predictive Abbrev Mode OFF"))
    (progn (global-pabbrev-mode)
	   (setq toggle-pabbrev-mode t)
	   (if (eq toggle-ac-auto-start-mode t)
	       (progn (setq ac-auto-start nil)
		      (setq toggle-ac-auto-start-mode nil)))
	   (message  "Global Predictive Abbrev Mode ON"))
))

;; -------------------------------------------------------------
;; Add custom menu options
;; -------------------------------------------------------------
(defun lig-add-to-options-menu ()
    (define-key-after menu-bar-options-menu [hs-mode]
    (menu-bar-make-mm-toggle hs-org/minor-mode
			   "Hide/Show mode"
			   "Hide & Show blocks of code")
      'custom-separator)
  (define-key-after menu-bar-options-menu [lock-mode]
    (menu-bar-make-mm-toggle dedicated-mode
			   "Dedicated mode"
			   "Emacs will not select files into that window")
      'hs-mode)
  (define-key-after menu-bar-options-menu [pair-mode]
    (menu-bar-make-mm-toggle pair-mode
			   "Pairing mode"
			   "Automatic insertion of corresponding ( { [ .")
      'lock-mode)
  (define-key-after menu-bar-options-menu [pabbrev-mode]
    (menu-bar-make-mm-toggle toggle-pabbrev-mode
			   "Predictive Abbreviation mode"
			   "Toggle ON/OFF Pabbrev mode")
      'pair-mode)
  (define-key-after menu-bar-options-menu [ac-mode]
    (menu-bar-make-mm-toggle toggle-ac-auto-start-mode
			     "AutoCompletion mode"
			     "Toggle ON/OFF AutoCompletion mode")
      'pabbrev-mode)
  (define-key-after menu-bar-options-menu [lig-separator]
    '(menu-item "--") 'ac-mode)
)

(lig-add-to-options-menu)


;; -------------------------------------------------------------
;; HideShowVis
;; -------------------------------------------------------------
(require 'hideshow-org)

(autoload 'hideshowvis-enable "hideshowvis" "Highlight foldable regions")

;; Not including this for two reasons:
;; 1. It is replacing Lisp-mode and Magik-mode menu
;; with Hide/show menu - needs time to fix it - TODO
;; 2. In Magik-mode - locking up my mashine   - TODO??
;(dolist (hook (list 'emacs-lisp-mode-hook
;                     'magik-mode-hook
;    ))
;   (add-hook hook 'hideshowvis-enable))

  (define-fringe-bitmap 'hs-marker [0 24 24 126 126 24 24 0])

  (defcustom hs-fringe-face 'hs-fringe-face
    "*Specify face used to highlight the fringe on hidden regions."
    :type 'face
    :group 'hideshow)

  (defface hs-fringe-face
    '((t (:foreground "#888" :box (:line-width 2 :color "grey75" :style released-button))))
    "Face used to highlight the fringe on folded regions"
    :group 'hideshow)

  (defcustom hs-face 'hs-face
    "*Specify the face to to use for the hidden region indicator"
    :type 'face
    :group 'hideshow)

  (defface hs-face
    '((t (:background "#ff8" :box t)))
    "Face to hightlight the ... area of hidden regions"
    :group 'hideshow)

  (defun display-code-line-counts (ov)
    (when (eq 'code (overlay-get ov 'hs))
      (let* ((marker-string "*fringe-dummy*")
             (marker-length (length marker-string))
             (display-string (format ".(%d)..." (count-lines (overlay-start ov) (overlay-end ov))))
             )
        (overlay-put ov 'help-echo "Hiddent text. C-c,= to show")
        (put-text-property 0 marker-length 'display (list 'left-fringe 'hs-marker 'hs-fringe-face) marker-string)
        (overlay-put ov 'before-string marker-string)
        (put-text-property 0 (length display-string) 'face 'hs-face display-string)
        (overlay-put ov 'display display-string)
        )))

  (setq hs-set-up-overlay 'display-code-line-counts)


;; -------------------------------------------------------------
;; Modified core Emacs since it isn't alowing to create a new file
;; -------------------------------------------------------------

(defun find-file (filename &optional wildcards)
  "Edit file FILENAME.
Switch to a buffer visiting file FILENAME,
creating one if none already exists.
Interactively, the default if you just type RET is the current directory,
but the visited file name is available through the minibuffer history:
type M-n to pull it into the minibuffer.

You can visit files on remote machines by specifying something
like /ssh:SOME_REMOTE_MACHINE:FILE for the file name.  You can
also visit local files as a different user by specifying
/sudo::FILE for the file name.
See the Info node `(tramp)Filename Syntax' in the Tramp Info
manual, for more about this.

Interactively, or if WILDCARDS is non-nil in a call from Lisp,
expand wildcards (if any) and visit multiple files.  You can
suppress wildcard expansion by setting `find-file-wildcards' to nil.

To visit a file without any kind of conversion and without
automatically choosing a major mode, use \\[find-file-literally]."
  (interactive
   (find-file-read-args "Find file: " nil
                        ;;(confirm-nonexistent-file-or-buffer) ;LIG
            ))
  (let ((value (find-file-noselect filename nil nil wildcards)))
    (if (listp value)
    (mapcar 'switch-to-buffer (nreverse value))
      (switch-to-buffer value))))

;; -------------------------------------------------------------
;; MISC custom settings
;; -------------------------------------------------------------
(setq scroll-step 4)
(setq default-indicate-empty-lines t)

(ido-mode 'buffer)
(setq ido-enable-flex-matching t)

;; -------------------------------------------------------------
;; Normally this saves customisations in your init file, `~/.emacs'.
;; If you wish, you can save customisations in another file instead.  To
;; make this work, your `~/.emacs' should set `custom-file' to the name of
;; that file. This is just to redirect saved customisations
;; -------------------------------------------------------------
 (setq custom-file "~/.emacs-custom")
 (load custom-file)


;;== !Leave this at the end! ===========
;;--------------------------------------------------------------
;; Save Emacs Desktop buffers
;;--------------------------------------------------------------
;   You can use the Desktop library to save the state of Emacs
;   from one session to another. Saving the state means that Emacs
;   starts up with the same set of buffers, major modes, buffer
;   positions, and so on that the previous Emacs session had.
;
;; Provides standard open-file-dialog in Win machines
;;
(require 'dlgopen)
(require 'desktop)

(add-hook 'kill-emacs-hook
                '(lambda ()
                   (desktop-truncate search-ring 3)
                   (desktop-truncate regexp-search-ring 3)))



(message (concat "....custom-lig loaded " (emacs-uptime)))

(provide 'custom_lig)
