;; Experimental interface to Rational cleartool.
;;
;; Provides User commands:
;;   cleartool   - start a cleartool session prompting for buffer name.
;;
;;   clearman    - interface to cleartool man Manual pages.

;; Current issues are:
;; clearman only works after a cleartool session is started
;; and Man has been run in the cleartool buffer.

;;Man page modifications for clearcase man
(require 'man)
(defvar man-prog manual-program)
(make-variable-buffer-local 'manual-program)

(defadvice man (before manual-program-variable-propagation activate compile)
  "Ensure that the buffer local variable manual-program gets propagated.
This is useful for ClearCase man pages where a different man command is used."
  (setq man-prog manual-program))

(defun manual-program-variable-propagation ()
  "Set manual-program from man-prog variable."
  (setq manual-program man-prog))

(add-hook 'Man-mode-hook 'manual-program-variable-propagation)

(defun clearman (man-args)
  "Shortcut for running cleartool man"
  (interactive
   (list (let* ((default-entry (Man-default-man-entry)))
	   (read-string
	    (format "Manual entry%s: "
		    (if (string= default-entry "")
			""
		      (format " (default %s)" default-entry)))))))
  (save-excursion
    (let ((Man-notify-method 'pushy)
	  (man-prog "cleartool man")
	  (manual-program "cleartool man"))
      (man "cleartool"))
;;    (let ((Man-notify-method 'pushy))
;;	(cond ((buffer-live-p "*Man cleartool*")
;;	       t)
;;	      ((buffer-live-p "*cleartool shell*")
;;	       (set-buffer "*cleartool shell*")
;;	       (man "cleartool"))
;;	      (t
;;	       (cleartool)
;;	       (set-buffer "*cleartool shell*")
;;	       (man "cleartool")
;;	       )))
    (set-buffer "*Man cleartool*")
    
    (man man-args)))

(defun cleartool (&optional output-buffer command error-buffer)
  "Start cleartool shell"
  (interactive)
  (let ((process-environment (append '("ATRIA_NO_BOLD=1"    ;;Otherwise output attempts to use extend font features in the shell
                                       "ATRIA_FORCE_GUI=1")
                                     ;;; emacs is a GUI, right? :-)
                                     process-environment))
	(command (or command (if (eq system-type 'windows-nt)
				 "u:/swdev/users/andrewma/Emacs/cts_shell.exe"
			       "cleartool")))
	(output-buffer (or output-buffer "*cleartool shell*")))

    (and (eq system-type 'windows-nt)
	 (setq process-environment (append (list (concat "ATRIAHOME="
							 (w32-short-file-name "c:\\Program Files\\Rational\\ClearCase")))
					   process-environment)))
    (setq command (concat command " &"))

    (shell-command command output-buffer error-buffer)
    (set-buffer output-buffer)
    (setq manual-program "cleartool man")))

(provide 'cleartool)
