;;; site-start.el - emacs startup file.

;;; This file, if it exists in the emacs load path is loaded before
;;; the users' .emacs file.

;;; We use it to point emacs at its info, etc and executable files.

(if (< emacs-major-version 20)
    (load (concat (file-name-directory load-file-name) "site-start-19.el")
	  nil t))

(defvar Info-default-directory-list)
(defvar sw-development nil
  "*Load Smalworld Development Packages.")

(defun sw-sitename ()
  (condition-case nil
      (save-excursion
	(set-buffer (get-buffer-create " *sw sitename temp buffer*"))
	(erase-buffer)
	(call-process "sitename" nil t nil)
	(prog1
	    (buffer-string)
	  (kill-buffer (current-buffer))))
    (error "")))


(let ((sw (getenv "SMALLWORLD_GIS"))
      (site (sw-sitename)))
  (cond ((or (string-match "^Burleigh" site)
	     (string-match "^Brunswick" site))
	 (setq sw-development t))

	;; Not at a development site.

	((eq system-type 'windows-nt)
	;; Windows uses gis.exe to setup the environment for Emacs
	 nil)
	(sw
	 ;; SMALLWORLD_GIS is defined and we are on UNIX.
	 ;; attempt to fix up various variables to enable the installed Smallworld product.
	 (or (string-match "/$" sw)
	     (setq sw (concat sw "/")))
	 (setq sw (concat sw "emacs/")
	       Info-default-directory-list (cons (concat sw "share/info") Info-default-directory-list)
	       data-directory (concat sw "share/etc/")
	       doc-directory  (concat sw "share/etc/")
	       exec-directory (concat sw (getenv "HOST_OS") "." (getenv "HOST_ARCH") "/etc/")
	       exec-path      (reverse (cons exec-directory (reverse exec-path)))))))


(provide 'site-start)

;;; site-start.el ends here
