;;; magik.el -- mode for editing Magik + some utils.

(eval-when-compile (require 'cl)
		   (and (> (string-to-int emacs-version) 19) (require 'speedbar))
		   (require 'imenu)
		   (require 'misc-sw))
(require 'font-lock)

(require 'sw-help)
(require 'magik-template)
(require 'indent-magik)
(require 'pragma)

;; compiler: variable declarations
(defvar gis-buffer)
(defvar buffer-file-coding-system nil) ;only relevent for Emacs 19 and earlier

(defgroup magik nil
  "Customise Magik Language group."
  :group 'smallworld
  :group 'languages)

(defconst magik-version "$Revision: 1.43 $")

(defcustom magik-mode-hook nil
  "*Hook to run after Magik mode is set."
  :group 'magik
  :type  'hook)

(defcustom transmit-magik-debug-p nil
  "*If t, \"#DEBUG\" patterns get stripped out of things being transmitted to magik"
  :group 'magik
  :type  'boolean)

(defcustom magik-under-as-char t
  "*Non-nil means that the _ (underline) should be treated as word char."
  :group 'magik
  :type  'boolean)

(defcustom magik-mark-method-exchange t
  "*If t, `magik-mark-method' (\\[magik-mark-method]) leaves point at end of marked method region.
nil leaves point at start of marked method region.
Standard Emacs marking commands like `mark-paragraph' and `mark-sexp'
(on keys \\[mark-paragraph] and \\[mark-sexp] respectively)
normally leave point at the start of the region.
But to keep most users happy, this is defaulted to t as this is
what most Smallworld users expect.
Users can also swap the point and mark positions using \\[exchange-point-and-mark]."
  :group 'magik
  :type  'boolean)

(defcustom magik-work-buffer nil
  "*Name of Magik buffer to be used for development work."
  :group 'magik
  :type  'string)

(defcustom magik-auto-abbrevs t
  "*User option which enables magik abbreviation expansion automatically"
  :group 'magik
  :type  'boolean)

(defcustom magik-sw-menu-file-max-length 90
  "*The maximum length of the displayed path in the SW -> Magik Files submenu."
  :group 'magik
  :type  'integer)

(defvar magik-mode-map (make-sparse-keymap)
  "Keymap for Magik files")

(defvar magik-menu nil
  "Keymap for the Magik buffer menu bar")

(easy-menu-define magik-menu magik-mode-map
  "Menu for Magik Mode."
  `(,resources-magik-menu
    [,resources-magik-menu-transmit-method   magik-transmit-method         :active (sw-buffer-mode-list 'gis-mode)
	 				                                   :keys "f7, f2 f7, f2 m"]
    [,resources-magik-menu-transmit-region   magik-transmit-region         :active (sw-buffer-mode-list 'gis-mode)
					                                   :keys "f8, f2 f8, f2 r"]
    [,resources-magik-menu-transmit-buffer   magik-transmit-buffer         :active (sw-buffer-mode-list 'gis-mode)
					                                   :keys "f2 b"]
    [,resources-magik-menu-transmit-chunk    magik-transmit-$-chunk        :active (sw-buffer-mode-list 'gis-mode)
					                                   :keys "f2 $"]
    [,resources-magik-menu-transmit-thing    magik-transmit-thing          :active (sw-buffer-mode-list 'gis-mode)
					                                   :keys "f2 RET"]
    "---"
    [,resources-magik-menu-work-copy-region  magik-copy-region-to-buffer   :active t :keys "f4 r"]
    [,resources-magik-menu-work-copy-method  magik-copy-method-to-buffer   :active t :keys "f4 m"]
    [,resources-magik-menu-work-set-buffer   magik-set-work-buffer-name    :active t :keys "f4 n"]
    "---"
    [,resources-magik-menu-electric-template explicit-electric-magik-space :active t :keys "f2 SPC"]
    [,resources-magik-menu-mark-method       magik-mark-method             :active t :keys "C-M-h, f9"]
    [,resources-magik-menu-copy-method       magik-copy-method             :active t :keys "f4 c, f6"]
    [,resources-magik-menu-compare-methods   magik-compare-methods         :active t :keys "f4 w"]
    [,resources-magik-menu-ediff-methods     magik-ediff-methods           :active t :keys "f4 e"]
    "---"
    [,resources-magik-menu-add-debug         magik-add-debug-statement     :active t :keys "f4 s"]
    [,resources-magik-menu-trace             magik-trace-curr-statement    :active t :keys "f2 t"]
    [,resources-magik-menu-complete          magik-symbol-complete          :active (sw-buffer-mode-list 'gis-mode) :keys "f4 f4"]
    [,resources-deep-print-menu-start        deep-print                     :active (and (fboundp 'deep-print)
											 (sw-buffer-mode-list 'gis-mode))
					                                   :keys "f2 x"]
    "---"
    [,resources-dev-tools-menu-oi            dev-tools-object-inspector     :active (sw-buffer-mode-list 'gis-mode) :keys "f4 o, f2 X"]
    [,resources-dev-tools-menu-vsd-method    dev-tools-vsd-transmit-method :active (and (or (not magik-method-name-mode)
											    (not (equal magik-method-name "")))
											(sw-buffer-mode-list 'gis-mode)) :keys "f4 d"]
    "---"
    [,resources-magik-menu-heading           magik-heading                 :active t :keys "f2 h"]
    [,resources-magik-menu-comment           magik-comment-region          :active t :keys "f2 #"]
    [,resources-magik-menu-uncomment         magik-uncomment-region        :active t :keys "f2 ESC #"]
    [,resources-magik-menu-fill              fill-magik-public-comment     :active t :keys "f2 q"]
    "---"
    (,resources-magik-menu-toggle
     [,resources-magik-menu-toggle-name-mode      magik-method-name-mode
						  :active t
						  :style toggle
						  :selected magik-method-name-mode]
     [,resources-magik-menu-toggle-electric-mode  electric-magik-mode
						  :active t
						  :keys "f2 e"
						  :style toggle
						  :selected electric-magik-mode]
     [,resources-magik-menu-toggle-debug          toggle-transmit-magik-debug-p
						  :active t
						  :style toggle
						  :selected transmit-magik-debug-p]
     [,resources-magik-menu-toggle-mark-exchange  magik-mark-method-exchange-mode
						  :active t
						  :style toggle
						  :selected magik-mark-method-exchange]
     ,resources-magik-menu-options
     [,resources-magik-menu-option-eom-mode-on    (magik-transmit-method-eom-mode 'end)
						  :active t
						  :style radio
						  :selected (eq magik-transmit-method-eom-mode 'end)]
     [,resources-magik-menu-option-eom-mode-off   (magik-transmit-method-eom-mode nil)
						  :active t
						  :style radio
						  :selected (eq magik-transmit-method-eom-mode nil)]
     [,resources-magik-menu-option-eom-mode-repeat (magik-transmit-method-eom-mode 'repeat)
						  :active t
						  :style radio
						  :selected (eq magik-transmit-method-eom-mode 'repeat)
						  ])
    [,resources-menu-sw-customize            magik-customize               t]
    [,resources-magik-menu-language-help     magik-language-help           t]
    [,resources-menu-sw-help                 magik-help                    t]))

(defvar magik-mode-abbrev-table nil
  "Abbrev table in use in Magik-mode buffers")

(defvar magik-mode-syntax-table nil
  "Syntax table in use in Magik-mode buffers.")

(defvar magik-imenu-expression
  `(
    ;(nil
    ; "^\\s-*\\(_abstract\\(\n\\|\\s-\\)+\\)?\\(_private\\(\n\\|\\s-\\)+\\)?\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 7)
    (,resources-magik-imenu-methods
     "^\\s-*_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 1)
    (,resources-magik-imenu-iterators
     "^\\s-*\\(_abstract\\(\n\\|\\s-\\)+\\)?\\(_private\\(\n\\|\\s-\\)+\\)?_iter\\(\n\\|\\s-\\)+_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 6)
    (,resources-magik-imenu-private
     "^\\s-*_private\\(\n\\|\\s-\\)+\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 4)
    (,resources-magik-imenu-abstract
     "^\\s-*_abstract\\(\n\\|\\s-\\)+\\(_private\\(\n\\|\\s-\\)+\\)?\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 6)
    (,resources-magik-imenu-display
     "_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\(show\\|write\\|print\\|debug_print\\|trace\\)\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 1)
    (,resources-magik-imenu-new
     "_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\(new\\|init\\)\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" magik-imenu-method-name 1)
    (,resources-magik-imenu-procedures
     "\\b_\\sw+\\(\n\\|\\s-\\)+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\s-*<<\\(\n\\|\\s-\\)*_proc\\s-*(" 2) ;unamed, use variable assignment
    (,resources-magik-imenu-procedures
     "_proc\\s-*\\(@\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\s-*(" magik-imenu-method-name 1) ;named using @
    (,resources-magik-imenu-conditions
     "^\\s-*condition.define_condition([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 1)
    (,resources-magik-imenu-property
     "^\\s-*\\(.+\\)\.def\\(\\|ine\\)_property([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 3)
    (,resources-magik-imenu-shared-variables
     "^\\s-*\\(.+\\)\.define_shared_variable([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 2)
    (,resources-magik-imenu-shared-constants
     "^\\s-*\\(.+\\)\.define_shared_constant([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 2)
    (,resources-magik-imenu-slot-access
     "^\\s-*\\(.+\\)\.define_slot_access([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 2) ; define_slot_externally_* rarely used.
    (,resources-magik-imenu-pseudo-slots
     "^\\s-*\\(.+\\)\.define_pseudo_slot([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 2) ; define_slot_externally_* rarely used.
    (,resources-magik-imenu-mixins
     "^\\s-*def_mixin([ \t\n]*:\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 1)
    (,resources-magik-imenu-binary-operator
     "^\\s-*define_binary_operator_case([ \t\n]*:\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 1)
    (,resources-magik-imenu-arrays
     "^\\s-*_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\s-*?\\[" magik-imenu-method-name 1)
    (,resources-magik-imenu-exemplars
     "^\\s-*def_slotted_exemplar([ \t\n]*:\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 1) ;;def_indexed_exemplar very rarely used. def_enumeration not used.
    (,resources-magik-imenu-globals
     "^\\s-*_global\\(\n\\|\\s-\\)+\\(_constant\\(\n\\|\\s-\\)+\\)?\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)" 4)
    (,resources-magik-imenu-package
     "^\\s-*_package[ \t\n]*\\(\\sw+\\)" 1))
  "Imenu expression for Magik mode.  See `magik-imenu-create-index-function'.")

;;; Font-lock configuration
;;
;; Do not use :inherit because we want Emacs 20 support.
;;
;; On Windows Emacsen 21 and earlier, there is a GDI Object based memory leak.
;; This has been tracked down to creating italic forms of the font.
;; However, bold italic forms do not appear affected,
;; so all color based italic fonts are also made bold too.
(defgroup magik-faces nil
  "Fontification colours for Magik."
  :group 'magik)

(defface magik-font-lock-keyword-operators-face
  '((((type tty) (class color)) (:foreground "cyan" :weight bold))
    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
    (((class color) (background light)) (:foreground "Sky Blue"))
    (((class color) (background dark)) (:foreground "Sky Blue"))
    (t (:bold t)))
  "Font-lock Face to use when displaying Magik operator keywords.

Based upon `font-lock-keyword-face'"
  :group 'magik-faces)

(defface magik-font-lock-keyword-statements-face
  '((((type tty) (class color)) (:foreground "cyan" :weight bold))
    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
    (((class color) (background light)) (:foreground "Steel Blue"))
    (((class color) (background dark)) (:foreground "Steel Blue"))
    (t (:bold t)))
  "Font-lock Face to use when displaying Magik statement keywords.

Based upon `font-lock-keyword-face'"
  :group 'magik-faces)

(defface magik-font-lock-keyword-loop-face
  '((((type tty) (class color)) (:foreground "cyan" :weight bold))
    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
    (((class color) (background light)) (:foreground "Dark Sea Green"))
    (((class color) (background dark)) (:foreground "Dark Sea Green"))
    (t (:bold t)))
  "Font-lock Face to use when displaying Magik loop keywords.

Based upon `font-lock-keyword-face'"
  :group 'magik-faces)

(defface magik-font-lock-keyword-arguments-face
  '((((type tty) (class color)) (:foreground "cyan" :weight bold))
    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
    (((class color) (background light)) (:foreground "Green"))
    (((class color) (background dark)) (:foreground "Green"))
    (t (:bold t)))
  "Font-lock Face to use when displaying Magik argument keywords.

Based upon `font-lock-keyword-face'"
  :group 'magik-faces)

(defface magik-font-lock-keyword-variable-face
  '((((type tty) (class color)) (:foreground "cyan" :weight bold))
    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
    (((class color) (background light)) (:foreground "Yellow Green"))
    (((class color) (background dark)) (:foreground "Green Yellow"))
    (t (:bold t)))
  "Font-lock Face to use when displaying Magik variable keywords.

Based upon `font-lock-keyword-face'"
  :group 'magik-faces)

(defface magik-font-lock-keyword-obsolete-face
  '((((type tty) (class color)) (:underline t :foreground "red"))
    (((class color) (background light)) (:underline t :foreground "Red" :bold t))
    (((class color) (background dark)) (:underline t :foreground "Pink" :bold t))
    (t (:inverse-video t :bold t)))
  "Font-lock Face to use when displaying obsolete Magik keywords.

Based upon `font-lock-warning-face'"
  :group 'magik-faces)

(defface magik-font-lock-boolean-face
   '((((type tty) (class color)) (:foreground "yellow" :weight light))
    (((class grayscale) (background light))
     (:foreground "Gray90" :bold t :italic t))
    (((class grayscale) (background dark))
     (:foreground "DimGray" :bold t :italic t))
    (((class color) (background light)) (:foreground "DarkGoldenrod"))
    (((class color) (background dark)) (:foreground "LightGoldenrod"))
    (t (:bold t :italic t)))
  "Font-lock Face to use when displaying boolean and kleenean references.

Based upon `font-lock-variable-name-face'"
  :group 'magik-faces)

(defface magik-font-lock-pragma-face
  '((((type tty) (class color)) (:bold t :foreground "blue" :weight light))
    (((class grayscale) (background light)) (:foreground "LightGray" :bold t))
    (((class grayscale) (background dark)) (:foreground "DimGray" :bold t))
    (((class color) (background light)) (:bold t :foreground "Orchid"))
    (((class color) (background dark)) (:bold t :foreground "LightSteelBlue"))
    (t (:bold t)))
  "Font-lock Face to use when displaying pragma directives.

Based upon `font-lock-builtin-face'"
  :group 'magik-faces)

(defface magik-font-lock-procedure-face
   '((((type tty) (class color)) (:foreground "blue" :weight bold))
    (((class color) (background light)) (:italic t :foreground "Blue"        :bold t)) ;originally just italic
    (((class color) (background dark)) (:italic t :foreground "LightSkyBlue" :bold t))
    (t (:inverse-video t :bold t)))
  "Font-lock Face to use when displaying procedure calls.

Based upon `font-lock-function-name-face'"
  :group 'magik-faces)

(defface magik-font-lock-slot-face
  '((((type tty) (class color)) (:foreground "yellow" :weight light))
    (((class grayscale) (background light))
     (:foreground "Gray90" :bold t :italic t))
    (((class grayscale) (background dark))
     (:foreground "DimGray" :bold t :italic t))
    (((class color) (background light)) (:italic t :foreground "DarkGoldenrod" :bold t)) ;originally just italic
    (((class color) (background dark)) (:italic t :foreground "LightGoldenrod" :bold t))
    (t (:bold t :italic t)))
  "Font-lock Face to use when displaying slots.

Based upon `font-lock-variable-name-face'"
  :group 'magik-faces)

(defface magik-font-lock-symbol-face
  '((((type tty) (class color)) (:foreground "magenta"))
    (((class grayscale) (background light))
     (:foreground "LightGray" :bold t :underline t))
    (((class grayscale) (background dark))
     (:foreground "Gray50" :bold t :underline t))
    (((class color) (background light)) (:foreground "CadetBlue"))
    (((class color) (background dark)) (:foreground "Aquamarine"))
    (t (:bold t :underline t)))
  "Font-lock Face to use when displaying symbols.

Based upon `font-lock-constant-face'"
  :group 'magik-faces)

(defface magik-font-lock-warning-face
   '((((type tty) (class color)) (:inverse-video t :foreground "red"))
    (((class color) (background light)) (:inverse-video t :foreground "Red" :bold t))
    (((class color) (background dark)) (:inverse-video t :foreground "Pink" :bold t))
    (t (:inverse-video t :bold t)))
  "Font-lock Face to use when displaying warning statements.

Based upon `font-lock-warning-face'"
  :group 'magik-faces)

(defface magik-font-lock-write-face
  '((((type tty) (class color)) (:foreground "red"))
    (((class color) (background light)) (:italic t :foreground "Red" :bold t))
    (((class color) (background dark)) (:italic t :foreground "Pink" :bold t))
    (t (:inverse-video t :bold t)))
  "Font-lock Face to use when displaying write() statements.

Based upon `font-lock-warning-face'"
  :group 'magik-faces)

(defvar magik-keyword-constants
  '("false" "true" "maybe" "unset" "constant")
  "List of keywords relating to constant values to highlight for font-lock.
The \"no_way\" constant is treated as a special case in this Magik mode
because it does not have an _ preceding like all the other Magik keywords.")

(defvar magik-keyword-operators
  '("and" "andif" "div" "is" "isnt" "cf" "mod" "not" "or" "orif" "xor")
  "List of keywords relating to operators to highlight for font-lock")

(defvar magik-keyword-class
  '("self" "super" "clone")
  "List of keywords relating to exemplars to highlight for font-lock")

(defvar magik-keyword-methods
  '("abstract" "private" "method" "endmethod" "primitive")
  "List of keywords relating to methods to highlight for font-lock")

(defvar magik-keyword-procedures
  '("proc" "endproc")
  "List of keywords relating to procedures to highlight for font-lock")

(defvar magik-keyword-statements
  '("block" "endblock" "catch" "throw" "endcatch"
    "if" "then" "elif" "else" "endif"
    "lock" "endlock" "protect" "locking" "protection" "endprotect"
    "try" "endtry" "when" "handling" "with" "using"
    "pragma" "package" "default" "thisthread")
  "List of keywords relating to statements to highlight for font-lock")

(defvar magik-keyword-loop
  '("iter" "continue" "finally" "for" "loop" "endloop" "loopbody" "over" "leave")
  "List of keywords relating to loops to highlight for font-lock")

(defvar magik-keyword-arguments
  '("gather" "scatter" "allresults" "optional" "return")
  "List of keywords relating to arguments to highlight for font-lock")

(defvar magik-keyword-variable
  '("dynamic" "global" "import" "local" "recursive")
  "List of keywords relating to variables to highlight for font-lock")

(defvar magik-keyword-obsolete
  '("concat" "case" "endcase" "otherwise" "void")
  "List of obsolete/unimplemented keywords to highlight for font-lock")

(defvar magik-other-keywords '(">>" "def_indexed_exemplar" "def_slotted_exemplar")
  "List of other Magik 'keywords'")

(defvar magik-warnings
  '("TODO" "DEBUG" "FIXME" "sys!slot" "sys!perform" "sys!perform_iter")
  "List of Magik Warnings")

(defcustom magik-font-lock-class-face 'font-lock-type-face
  "*Font-lock Face to use when displaying exemplars."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-method-face 'font-lock-function-name-face
  "*Font-lock Face to use when displaying method names and method and procedure keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keyword-operators-face 'magik-font-lock-keyword-operators-face
  "*Font-lock Face to use when displaying Magik operator keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keyword-statements-face 'magik-font-lock-keyword-statements-face
  "*Font-lock Face to use when displaying Magik statement keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keyword-loop-face 'magik-font-lock-keyword-loop-face
  "*Font-lock Face to use when displaying Magik loop keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keyword-arguments-face 'magik-font-lock-keyword-arguments-face
  "*Font-lock Face to use when displaying Magik argument keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keyword-variable-face 'magik-font-lock-keyword-variable-face
  "*Font-lock Face to use when displaying Magik variable keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keyword-obsolete-face 'magik-font-lock-keyword-obsolete-face
  "*Font-lock Face to use when displaying obsolete Magik keywords."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-boolean-face 'magik-font-lock-boolean-face
  "*Font-lock Face to use when displaying boolean and kleenean references."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-dynamic-face 'font-lock-variable-name-face
  "*Font-lock Face to use when displaying dynamic variables."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-pragma-face 'magik-font-lock-pragma-face
  "*Font-lock Face to use when displaying pragma directives."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-procedure-face 'magik-font-lock-procedure-face
  "*Font-lock Face to use when displaying procedure calls."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-slot-face 'magik-font-lock-slot-face
  "*Font-lock Face to use when displaying slots."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-symbol-face 'magik-font-lock-symbol-face
  "*Font-lock Face to use when displaying symbols."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-warning-face 'magik-font-lock-warning-face
  "*Font-lock Face to use when displaying warning statements."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-write-face 'magik-font-lock-write-face
  "*Font-lock Face to use when displaying write() statements."
  :group 'magik
  :type 'face)

(defcustom magik-font-lock-keywords-1
  (list
    (cons (concat "\\<no_way\\|_" (regexp-opt magik-keyword-constants t) "\\>") 'font-lock-constant-face)
    (cons (concat "\\<_"
		  (regexp-opt (append magik-keyword-operators
				      magik-keyword-class
				      magik-keyword-methods
				      magik-keyword-procedures
				      magik-keyword-statements
				      magik-keyword-loop
				      magik-keyword-arguments
				      magik-keyword-variable) t)
		  "\\>") 'font-lock-keyword-face)
    (cons (concat "\\<\\(" (mapconcat 'identity magik-other-keywords "\\|") "\\)\\>")
	  'font-lock-keyword-face))
  "Font lock setting for 1st level of Magik fontification.
Fontifies all Magik keywords in the same face except Magik
constants which use the `font-lock-constant-face' face."
   :group 'magik
   :type 'sexp)

(defcustom magik-font-lock-keywords-2
  (list
   '("\\b_method\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\.\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)"
     (1 magik-font-lock-classname-face)
     (3 magik-font-lock-methodname-face))
     ;(1 magik-font-lock-class-face)
     ;(3 magik-font-lock-method-face))
   '("\\<!\\sw+\\!\\>" .  magik-font-lock-dynamic-face)
   '("\\<:\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?" . magik-font-lock-symbol-face)
   '("\\<\\(write\\|print\\|debug_print\\)\\s-*(" 1 magik-font-lock-write-face)
    (list (concat "\\<\\("
		  (mapconcat 'identity magik-warnings "\\|")
		  "\\)")
	  0 'magik-font-lock-warning-face t)
   '("^\\s-*##.*$" 0 font-lock-doc-face t)
   )
  "Font lock setting for 2nd level of Magik fontification.
Fontifies certain Magik language features like symbols, dynamics but does
NOT fontify ANY Magik Keywords."
  :group 'magik
  :type 'sexp)

(defcustom magik-font-lock-keywords-3
  (append magik-font-lock-keywords-1 magik-font-lock-keywords-2 nil)
  "Combines 1st and 2nd level fontification levels.
See `magik-font-lock-keywords-1' and `magik-font-lock-keywords-2'."
   :group 'magik
   :type 'sexp)

(defcustom magik-font-lock-keywords-4
  (append
   magik-font-lock-keywords-2
   (list
    (cons (concat "\\<no_way\\|_" (regexp-opt magik-keyword-constants t) "\\>") 'font-lock-constant-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-constants  t) "\\>") 'font-lock-constant-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-operators  t) "\\>") 'magik-font-lock-keyword-operators-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-class      t) "\\>") 'magik-font-lock-class-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-methods    t) "\\>") 'magik-font-lock-method-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-procedures t) "\\>") 'magik-font-lock-procedure-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-statements t) "\\>") 'magik-font-lock-keyword-statements-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-loop       t) "\\>") 'magik-font-lock-keyword-loop-face)    ;; warnings
    (list (concat "\\<\\(" (mapconcat 'identity magik-warnings "\\|") "\\)\\>") 1 'magik-font-lock-warning-face t)

    (cons (concat "\\<_" (regexp-opt magik-keyword-arguments  t) "\\>") 'magik-font-lock-keyword-arguments-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-variable   t) "\\>") 'magik-font-lock-keyword-variable-face)
    (cons (concat "\\<_" (regexp-opt magik-keyword-obsolete   t) "\\>") 'magik-font-lock-keyword-obsolete-face)
    ;; other "keywords"
    (cons (concat "\\<\\(" (mapconcat 'identity magik-other-keywords "\\|") "\\)\\>") 'font-lock-keyword-face)
    '("^_pragma\\s-*\\(([^)]*)\\)" 1 magik-font-lock-pragma-face)
    ;; methods
    '("\\(\\sw\\|\\s$\\)\\.\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\(\\s-*(\\)" 2 magik-font-lock-method-face)
    ;; procedures
    '("\\<\\(\\sw+\\)\\(\\s-*(\\)" 1 magik-font-lock-procedure-face)
    '("^\\(def_slotted_exemplar\\|def_indexed_exemplar\\)\\>" 0 magik-font-lock-class-face t)
    '("^\\(\\sw+\\)\\.define_\\(shared_constant\\|shared_variable\\|slot_access\\)\\>" 1 magik-font-lock-class-face)
    '("\\Sw\\(\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\>" 1 magik-font-lock-slot-face)
    '("\\<\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\?\\>" 0 magik-font-lock-boolean-face t)
    '("_for\\s-+\\(\\sw+\\)" 1 font-lock-variable-name-face) ;_for loop variable
    '("@\\sw+" 0 font-lock-constant-face t)
    ))
  "Font lock setting for 4th level of Magik fontification.
As 1st level but also fontifies all Magik keywords according their
different classifications. ie. loop keywords are fontified in the same face."
   :group 'magik
   :type 'sexp)

(defcustom magik-font-lock-keywords magik-font-lock-keywords-1
  "Default fontification of Magik."
  :group 'magik
  :type 'sexp)

;;Could have coded the following to be generic but this is more maintainable.
;;Have added support for 3d arrays although there are none in Core at present.
(defvar magik-goto-class-method-array-alist
  '(("[]"      . "\\[[^\],]+\\]\\s-*$")
    ("[]<<"    . "\\[[^\],]+\\]\\s-*<<")
    ("[]^<<"   . "\\[[^\],]+\\]\\s-*\\^<<")
    ("[,]"     . "\\[[^\],]+,[^\]]+\\]\\s-*$")
    ("[,]<<"   . "\\[[^\],]+,[^\]]+\\]\\s-*<<")
    ("[,]^<<"  . "\\[[^\],]+,[^\]]+\\]\\s-*\\^<<")
    ("[,,]"    . "\\[[^\],]+,[^\],]+,[^\]]+\\]\\s-*$")
    ("[,,]<<"  . "\\[[^\],]+,[^\],]+,[^\]]+\\]\\s-*<<")
    ("[,,]^<<" . "\\[[^\],]+,[^\],]+,[^\]]+\\]\\s-*\\^<<"))
  "Alist to help searching for array type methods.")

(defvar magik-transmit-method-eom-alist
  (list (cons resources-magik-menu-option-eom-mode-off    nil)
	(cons resources-magik-menu-option-eom-mode-on     'end)
	(cons resources-magik-menu-option-eom-mode-repeat 'repeat))
  "Alist of options for `magik-transmit-method-eom-mode'.")

(defcustom magik-transmit-method-eom-mode nil
  "Variable storing setting of \\[magik-transmit-method-eom-mode]."
  :group  'magik
  :type  (list 'choice
	       (list 'const ':tag resources-magik-menu-option-eom-mode-off nil)
	       (list 'const ':tag resources-magik-menu-option-eom-mode-on 'end)
	       (list 'const ':tag resources-magik-menu-option-eom-mode-repeat 'repeat)))

(defcustom magik-method-name-mode nil
  "Variable storing setting of \\[magik-method-name-mode].
This variable exists to allow a user to set the mode before the Smallworld
code is loaded."
;;Use of integers is a standard way of forcing minor modes on and off.
  :group 'magik
  :type '(choice (const :tag "On" 1)
		 (const :tag "Off" -1)))

(defvar magik-method-name ""
  "Variable storing method name at which point it in.
Used by \\[magik-method-name-mode].")

(defvar magik-method-name-set-text-function 'magik-method-name-set-text-properties
  "Function to use for setting the Mode line to include Method name.
Function takes two arguments BUFFER and METHOD.")

;;; Help
(defun magik-help ()
  "Display help on how to use the Magik Mode interface."
  (interactive)
  (sw-help-open sw-help-magik-id))

(defun magik-language-help ()
  "Display help about the Magik Language."
  (interactive)
  (sw-help-open sw-help-magik-language-id))

(defun magik-customize ()
  "Open Customization buffer for Magik Mode."
  (interactive)
  (customize-group 'magik))

;;; Functions

(defun expand-magik-abbrev ()
  (save-excursion
    (let*
        ((toks (progn (insert ? )  ;; so that the token closes!
                      (prog1
                          (tokenise-magik-region-no-eol (point-bol) (point))
                        (delete-backward-char 1))))
         (last-tok (car (last toks)))
         (last-tok-pos (cdr last-tok)))
      (backward-word 1)
      (if (and (eq (point) last-tok-pos)
               (/= (preceding-char) ?.))
          (insert ?_))
      (if (and (eq major-mode 'magik-mode)
               (looking-at "_else\\|_elif\\|_finally\\|_using\\|_with\\|_when\\|_protection\\|_end"))
          (magik-indent-command)))))

;;Actually only used by the Magik-Patch minor mode but we need a hook here
;;because a function must be referred to in font-lock-defaults.
(defvar magik-goto-code-function 'point-min
  "Function used to place point on the line immediately preceeding Magik code.")

(defun magik-goto-code ()
  "Goto start of code."
  (funcall magik-goto-code-function))

(defun magik-font-lock-fontify-buffer ()
  (let ((verbose (if (numberp font-lock-verbose)
		     (> (buffer-size) font-lock-verbose)
		   font-lock-verbose))
	(code-start (save-excursion (magik-goto-code))))
    (with-temp-message
	(when verbose
	  (format "Fontifying %s..." (buffer-name)))
      ;; Make sure we have the right `font-lock-keywords' etc.
      (unless font-lock-mode
	(font-lock-set-defaults))
      ;; Make sure we fontify etc. in the whole buffer.
      (save-restriction
	(widen)
	(condition-case nil
	    (save-excursion
	      (save-match-data
		(font-lock-fontify-region code-start (point-max) verbose)
		(font-lock-after-fontify-buffer)
		(setq font-lock-fontified t)))
	  ;; We don't restore the old fontification, so it's best to unfontify.
	  (quit (font-lock-unfontify-buffer))))
      ;; Make sure we undo `font-lock-keywords' etc.
      (unless font-lock-mode
	(font-lock-unset-defaults)))))

(defun magik-font-lock-unfontify-buffer ()
  ;; Make sure we unfontify etc. in the whole buffer.
  (save-restriction
    (widen)
    (font-lock-unfontify-region (save-excursion (magik-goto-code)) (point-max))
    (font-lock-after-unfontify-buffer)
    (setq font-lock-fontified nil)))

(defun magik-font-lock-fontify-region (beg end loudly)
  (let*
      ((modified (buffer-modified-p))
       (buffer-undo-list t)
       (inhibit-read-only t)
       (inhibit-point-motion-hooks t)
       (inhibit-modification-hooks t)
       deactivate-mark buffer-file-name buffer-file-truename
       (parse-sexp-lookup-properties font-lock-syntactic-keywords)
       (old-syntax-table (syntax-table))
       (code-start (save-excursion (magik-goto-code))))
    (unwind-protect
	(save-restriction
	  (widen)
	  ;; Use the fontification syntax table, if any.
	  (when font-lock-syntax-table
	    (set-syntax-table font-lock-syntax-table))
	  ;; check to see if we should expand the beg/end area for
	  ;; proper multiline matches
	  (when (and (boundp 'font-lock-multiline)
		     font-lock-multiline
		     (> beg code-start)
		     (get-text-property (1- beg) 'font-lock-multiline))
	    ;; We are just after or in a multiline match.
	    (setq beg (or (previous-single-property-change
			   beg 'font-lock-multiline)
			  code-start))
	    (goto-char beg)
	    (setq beg (line-beginning-position)))
	  (when (and (boundp 'font-lock-multiline) font-lock-multiline)
	    (setq end (or (text-property-any end (point-max)
					     'font-lock-multiline nil)
			  (point-max))))
	  (goto-char end)
	  (setq end (line-beginning-position 2))
	  (if (and (>= end code-start) (< beg code-start))
	      (setq beg code-start))
	  (when (and (>= beg code-start)
		     (>= end code-start))
	    ;; Now do the fontification.
	    (font-lock-unfontify-region beg end)
	    (when font-lock-syntactic-keywords
	      (font-lock-fontify-syntactic-keywords-region beg end))
	    (unless font-lock-keywords-only
	      (font-lock-fontify-syntactically-region beg end loudly))
	    (font-lock-fontify-keywords-region beg end loudly)))
      ;; Clean up.
      (set-syntax-table old-syntax-table))
    (if (and (not modified) (buffer-modified-p))
      (set-buffer-modified-p nil))))

(defun toggle-transmit-magik-debug-p (&optional arg)
  "Toggle transmission of #DEBUG statements in Magik code."
  (interactive "P")
  (setq transmit-magik-debug-p
	(if (null arg)
	    (not transmit-magik-debug-p)
	  (> (prefix-numeric-value arg) 0)))
  (message
   (if transmit-magik-debug-p
       resources-transmit-magik-debug-on
     resources-transmit-magik-debug-off)))

(defun magik-add-debug-statement ()
  "Add a debug statement at the current line of magik."
  (interactive)
  (let
      ((var (sw-find-tag-default))
       (pos (point))
       line
       col
       tb)
    (save-excursion
      (or var (error resources-magik-no-current-variable-error))
      (backward-method)
      (push-mark nil t)
      (while
	  (not (or (eobp)
		   (looking-at "^\\s-*\\(_abstract\\(\n\\|\\s-\\)+\\)?\\(_private\\(\n\\|\\s-\\)+\\)?\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)")))
	(forward-line 1))
      (setq tb (match-string-no-properties 6))
      (setq line (count-lines (point) pos)))
    (back-to-indentation)
    (setq col (current-column))
    (beginning-of-line)
    (indent-to col)
    (insert (format "#DEBUG show(\"%s:Line %d\",:%s,%s)\n" tb line var var))))

;; bound to CR in magik mode.
(defun magik-newline ()
  "Insert a newline and indent.  (To insert a newline and not indent, use C-j)."
  (interactive "*")
  (if (eq major-mode 'gis-mode)
      (error resources-magik-mode-is-gis-error))
  (if abbrev-mode (expand-abbrev))
  (if (save-excursion
        (back-to-indentation)
        (looking-at "[]})]\\|_else\\|_finally\\|_using\\|_with\\|_when\\|_protection\\|_end"))
      (magik-indent-command))
  (newline-and-indent))

(defun magik-indent-line ()
  "Indent the current line as Magik code."
  (let ((indent (calc-magik-indent))
	(pos (- (point-max) (point)))
	beg change)
    (beginning-of-line)
    (setq beg (point))
    (skip-chars-forward " \t")
    (setq change (- indent (current-column)))
    (if (zerop change)
	nil
      (delete-region beg (point))
      (indent-to indent))
    ;; if the initial point was within the inden, leave point at the indent,
    ;; otherwise back to where we where
    (if (> (- (point-max) pos) (point))
	(goto-char (- (point-max) pos)))))

;; bound to TAB in magik mode.
(defun magik-indent-command ()
  "Indent the current line as Magik code, or hop between pragma fields."
  (interactive "*")
  (let ((pragma-brackets (pragma-line-p)))
    (if (consp pragma-brackets)
	(electric-pragma-tab pragma-brackets)
      (magik-indent-line))))

(defun magik-method-name-type (name)
  "Return cons cell describing the method name parts (NAME . TYPE).
NAME is the method name root
TYPE is ^<<, <<, ( or empty string."
  (let ((type (if (> (length name) 2) (substring name -2) name)))
    (cond ((equal type "()")
	   (cons (substring name 0 -2) "("))
	  ((and (> (length name) 2) (equal (substring name -3) "^<<"))
	   (cons (substring name 0 -3) "^<<"))
	  ((equal type "<<")
	   (cons (substring name 0 -2) "<<"))
	  (t
	   (cons name "\r?\n")))))

(defun magik-goto-class-method (method &optional class)
  "Find the specified METHOD and CLASS in the current buffer.
If more than one definition is found in the buffer, you will be
given the opportunity to visit each definition.
Also the search string is added to isearch mode's regexp ring so that
you can use \\[isearch-forward-regexp] and use M-p to recall the search."
  (interactive
   (list
    (read-string (concat resources-magik-goto-method-prompt ": ") (current-word))
    (if current-prefix-arg
	(read-string (concat resources-magik-goto-class-prompt ": ")
		     (file-name-sans-extension (file-name-nondirectory (buffer-file-name)))))))

  (if class nil
    (setq class (file-name-sans-extension (file-name-nondirectory (buffer-file-name)))))
  (let* ((method-cons (magik-method-name-type method))
	 (method-root (car method-cons))
	 (method-type (cdr method-cons))
	 search-str)
    (cond
     ((string-equal class "<global>")
      ;; look for _global definitions
      (setq search-str (concat (regexp-quote method) "\\s-*<<"))
      (if (re-search-forward search-str nil t)
	  (magik-goto-class-method-loop search-str method)
	(error resources-magik-goto-no-m-error method)))
     ((eq (elt method 0) ?\[)
      ;; look for array definitions: [] []<< []^<< [,] [,]<< [,]^<<
      (setq search-str (cdr (assoc method magik-goto-class-method-array-alist)))
      (or search-str (error resources-magik-goto-no-m-error method))

      (setq search-str (concat class search-str))
      (if (re-search-forward search-str nil t)
	  (magik-goto-class-method-loop search-str method)
	(error resources-magik-goto-no-m-error method)))
     ((re-search-forward
       ;; look for an ordinary _method constructs.
       (setq search-str
	     (concat
	      "^"
	      "\\(_abstract\\s-+\\)?"
	      "\\(_private\\s-+\\)?"
	      "\\(_iter\\s-+\\)?"
	      "_method\\s-+"
	      class
	      "\\."
	      method-root "\\s-*" method-type))
       nil t)
      (magik-goto-class-method-loop search-str (concat class "." method)))
     ;; look for other method constructors.
     ((re-search-forward
       (setq search-str (concat "^" class
				".\\(define_\\(shared_constant\\|shared_variable\\|slot_access\\|property\\)\\|def_property\\)"
				"\\s-*([ \t\r\n]*:" method-root))
       nil t)
      (magik-goto-class-method-loop search-str (concat class "." method)))
     (t
      (error resources-magik-goto-no-m-in-c-error method class)))))

(defun magik-goto-class-method-loop (search-str arg)
  "Loop over subsequent definitions.
Adds string to `regexp-search-ring'. After wuiting this loop
you can use \\[isearch-forward-regexp] and use M-p to recall this search."
  ;;I would like to use the isearch functionality but I cannot work out
  ;;how to control isearch programmatically.
  (let ((continue-p t)
	(pt (point))
	(prompt       (concat (format resources-magik-goto-class-method-loop-prompt arg) " "))
	(start-prompt (concat (format resources-magik-goto-class-method-loop-start-prompt arg) " "))
	next)
    (save-excursion
      (setq next (re-search-forward search-str nil t)))
    (if (null next)
	nil ;only single definition found exit here.
      (beep)
      (isearch-update-ring search-str t)
      (while continue-p
	(cond ((null next)
	       ;;No additional definitions found
	       (if (not (y-or-n-p start-prompt))
		   (setq continue-p nil)
		 (beep)
		 (goto-char pt)
		 (setq continue-p t)))
	      ((y-or-n-p prompt)
	       (goto-char next))
	      (t
	       (setq continue-p nil)))
	(save-excursion
	  (setq next (re-search-forward search-str nil t))))
      (where-is 'isearch-forward-regexp))))

(defun backward-method (&optional noerror)
  "Put point at beginning of this method."
  (interactive)

  (when (re-search-backward "^\\s-*\\(_abstract\\(\n\\|\\s-\\)+\\)?\\(_private\\(\n\\|\\s-\\)+\\)?\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+" nil noerror)
    (while
	(and (not (bobp))
	     (progn
	       (forward-line -1)
	       (looking-at "_abstract\\|_private\\|_iter\\|_pragma\\|[ \t]*#\\|[ \t]*$"))))
    (while
	(not (looking-at "_method\\|_iter\\|_private\\|_abstract\\|_pragma"))
      (forward-line 1))
    t))

(defun forward-method (&optional noerror)
  "Put point at beginning of the next method."
  (interactive)
  (save-match-data
    (and (not (eobp))
	 (save-excursion
	   (beginning-of-line)
	   (looking-at "_abstract\\|_private\\|_iter\\|_method\\|_pragma\\|[ \t]*#\\|[ \t]*$"))
	 (forward-endmethod noerror))

    (when (re-search-forward "^\\s-*\\(_abstract\\(\n\\|\\s-\\)+\\)?\\(_private\\(\n\\|\\s-\\)+\\)?\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+" nil noerror)
      (forward-line 1)
      (while (and (not (bobp))
		  (progn
		    (forward-line -1)
		    (looking-at "_abstract\\|_private\\|_iter\\|_method\\|_pragma\\|[ \t]*#\\|[ \t]*$"))))
      (while
	  (not (looking-at "_method\\|_iter\\|_private\\|_abstract\\|_pragma"))
	(forward-line 1)))))

(defun forward-endmethod (&optional noerror)
  "Put point at beginning of line after next _endmethod (including $)."
  (interactive)
  (save-match-data
    (when (re-search-forward "^_endmethod\\s-*\\(\n\\$\\s-*\\)?$" nil noerror)
      (forward-line 1)
      t)))

(defun magik-mark-method (&optional nomsg)
  "Mark the current method.
Repeated commands extends the marked region to include the next method as well."
  (interactive)
  (if (eq last-command 'magik-mark-method)
      (let (pt)
	(or magik-mark-method-exchange (exchange-point-and-mark))
	(setq pt (mark))
	(forward-endmethod t)
	(set-mark (point))
	(goto-char pt))
    (forward-endmethod t)
    (push-mark (point) nomsg t)
    (backward-method t))
  (if magik-mark-method-exchange (exchange-point-and-mark))
  (mark))

(defun mark-method ()
  "DEPRECATED. Mark the current method.
This is an old function."
  ;; also returns the value of mark.
  (interactive)
  (let
      (ans)
    (beginning-of-line)
    (forward-endmethod)
    (backward-method)
    (setq ans (point))
    (push-mark nil t)
    (forward-endmethod)
    ans))
(put 'mark-method 'disabled t)

(defun magik-copy-method ()
  "Copy the current method into `kill-ring'.
If the last command was \\[magik-mark-method] then that region will be copied instead."
  (interactive)
  (if (and (eq last-command 'magik-mark-method) mark-active)
      (kill-new (buffer-substring (point) (mark)))
    (save-excursion
      (save-match-data
	(magik-mark-method)
	(kill-new (buffer-substring (point) (mark)))))))

(defun magik-function-convert (x)
  "Convert lisp object X type into an equivalent Magik object type."
  (cond ((eq x 'unset)
	 "_unset")
	((or (eq x 'false) (eq x nil))
	 "_false")
	((or (eq x 'true) (eq x t))
	 "_true")
	((stringp x)
	 (concat "\"" x "\""))
	((numberp x)
	 (number-to-string x))
	((symbolp x)
	 (concat ":" (symbol-name x)))
	(t
	 x)))

(defun magik-function (cmd &rest args)
"this function generates magik code from the supplied arguments.
e.g. (magik-function \"system.test\" \"file\" 'unset 4) returns the string
     system.test(\\\"file\\\", _unset, 4)"

  ;process arg types: nil, string, other...
  (setq args (mapcar 'magik-function-convert args))
  ;bring the command together adding commas between the arguments
  (concat cmd "(" (mapconcat 'identity args ", ") ")\n"))

(defun magik-gis-error-goto (&optional gis)
  "Goto the previous magik error."
  (interactive)
  (let ((gis (sw-get-buffer-mode gis
				 'gis-mode
				 resources-gis-enter-buffer
				 gis-buffer
				 'gis-buffer-alist-prefix-function))
	pt)
    (save-excursion
      (set-buffer gis)
      (goto-char (point-max))
      (forward-line -1)
      (cond ((equal (current-word) "True")
	     nil) ;;Code loading successful.
	    ((re-search-backward (concat "^\\*\\*\\*\\*.*" "on line" " \\([0-9]+\\)$")
				 (save-excursion (re-search-backward gis-prompt nil t)) t)
	     (setq pt (point)))
	    (t ;no "on line" errors found.
	     nil)))

    (if pt
	(progn
	  (pop-to-buffer gis)
	  (goto-char pt)
	  (gis-error-goto)))))

(defun magik-perform-replace-no-set-mark (from to regexp-flag)
  "like `perform-replace' but without setting the mark and without
`query' or `delimited' flags."
  (let ((literal (not regexp-flag))
	(search-function (if regexp-flag 're-search-forward 'search-forward)))
    (while (and (not (eobp))
                (funcall search-function from nil t))
      (replace-match to t literal))))

(defun magik-transmit-method-eom-mode (arg)
  "Toggle whether to move the cursor to the end of the method after tranmitting
If nil, leave point where it is,
If t or 'end, move point to end of method,
If 'repeat, move point to end of method on 2nd and subsequent uses of the command."
  (interactive
   (list
    (cdr (assoc (completing-read (concat "Transmit Method EOM Mode:" " ")
				 magik-transmit-method-eom-alist
				 nil t)
		magik-transmit-method-eom-alist))))
  (setq magik-transmit-method-eom-mode arg)

  (message
   (cond ((null magik-transmit-method-eom-mode)
	  resources-magik-transmit-method-eom-off)
	 ((eq magik-transmit-method-eom-mode 'end)
	  resources-magik-transmit-method-eom-on)
	 ((eq magik-transmit-method-eom-mode 'repeat)
	  resources-magik-transmit-method-eom-on-repeat))))

(defun magik-mark-method-exchange-mode (&optional arg)
  "Toggle whether the cursor is placed at the beginning or end of the marked region.
See `magik-mark-method-exchange' for more details."
  (interactive "P")
  (setq magik-mark-method-exchange
	(if (null arg)
	    (not magik-mark-method-exchange)
	  (> (prefix-numeric-value arg) 0)))
  (message
   (if magik-mark-method-exchange
       resources-magik-mark-method-exchange-on
     resources-magik-mark-method-exchange-off)))

(defun magik-transmit-buffer ()
  "Send the buffer to the process running in the buffer in the var, `gis-buffer'."
  (interactive)
  (magik-transmit-region (point-min) (point-max))
  (message resources-magik-code-loaded (or (buffer-file-name) (buffer-name))))
(defalias 'transmit-buffer-to-magik 'magik-transmit-buffer)

(defun magik-transmit-thing ()
  "Transmit the top-level Magik programming construct surrounding point.
The construct can be a method, a proc, a def_slotted_exemplar or whatever.
The rule is that the thing must start against the left margin."
  (interactive)
  (let
      ((original-point (point))
       (beg (point))
       (stack nil))
    (if (re-search-backward "^\\w" nil t)
        (progn
          (setq beg (point))
          (forward-line -1)
          (while
              (and (not (bobp))
                   (looking-at "[ \t]*#\\|_pragma\\|_private\\|_iter\\|_if\\|_over\\|_for\\|[ \t]*usage"))
            (setq beg (point))
            (forward-line -1))
          (goto-char beg)
          (while
              (and (not (eq (point) (point-max)))
                   (or (< (point) original-point)
                       stack))
            (dolist (tok (tokenise-magik-line))
              (cond
               ((assoc (car tok) magik-begins-and-ends)
                (push (car tok) stack))
               ((assoc (car tok) magik-ends-and-begins)
                (if (equal (cdr (assoc (car stack) magik-begins-and-ends)) (car tok))
                    (pop stack)
                  (goto-char (cdr tok))
                  (error resources-magik-expecting-token-error
                         (car tok)
                         (cdr (assoc (car stack) magik-begins-and-ends)))))))
            (forward-line))
          (if (< (point) original-point)
              (progn
                (goto-char original-point)
                (error resources-magik-transmit-error))
            (magik-transmit-region beg (point)))))
    (goto-char original-point)))
(defalias 'transmit-thing-to-magik 'magik-transmit-thing)

(defun magik-transmit-$-chunk ()
  "Send the current $ chunk to magik."
  ;;Would like to use prefix key for number of statements but that classhes
  ;;with use of prefix to identify GIS session to send to.
  (interactive)
  (save-excursion
    (save-match-data
      (let ((pt (point))
	    (end (or (re-search-forward "^\\$" nil t)
		     (goto-char (point-max)))))
	(goto-char pt)
	(magik-transmit-region
	 (or (re-search-backward "^\\$" nil t)
	     (goto-char (point-min)))
	 end)))))

(defun magik-transmit-method ()
  "Send the current method to magik.
The location of point is determined by `magik-transmit-method-eom-mode'.
If 'repeat, then repeated calls to this function behaves like this:
  First use of command will leave point where it is,
  Repeat use will move point to the end of last transmitted method.
If 'end, then point is left at the end of the method.
Otherwise, point is left where it is."
  (interactive)
  ;;DEBUG (message "this %s, last %s" this-command last-command)
  (if (eq last-command 'magik-transmit-method-first) (forward-endmethod))
  (let ((magik-mark-method-exchange nil)
	mark)
    (save-excursion
      (setq mark (magik-mark-method t))
      (magik-transmit-region (point) mark))
    (cond ((eq magik-transmit-method-eom-mode 'end)
	   (goto-char mark))
	  ((eq magik-transmit-method-eom-mode 'repeat)
	   ;;Use this-command and last-command to track repetition.
	   (cond ((eq last-command 'magik-transmit-method-nth)
		  (goto-char mark)
		  (setq this-command 'magik-transmit-method-nth))
		 ((eq last-command 'magik-transmit-method-first)
		  (goto-char mark)
		  (setq this-command 'magik-transmit-method-nth))
		 ((eq this-command 'magik-transmit-method)
		  (setq this-command 'magik-transmit-method-first))))
	  (t nil))
    mark))

(defalias 'transmit-method-to-magik 'magik-transmit-method)

(defun magik-transmit-region (beg end)
   "Send current region via a temp file to Magik in a shell, using load-file.
If this command is repeated before the previous file has been processed by Magik,
another file shall be written."
   (interactive "r")
   (magik-transmit-string (buffer-substring-no-properties beg end)
			  (save-excursion
			    (goto-char beg)
			    (beginning-of-line)
			    (magik-package-line))
			  (lambda (f) (magik-function "load_file" f 'unset (or (buffer-file-name) 'unset)))
			  (lambda (f) (magik-function "system.unlink" f 'false 'true))
			  beg))
(defalias 'transmit-region-to-magik 'magik-transmit-region)

(defun magik-package-line ()
  "Return the _package line if one exists."
  ;;Find package name stripping off any surrounding white-space like ^M characters
  ;;Start from the current line and search backwards.
  ;;We are not usually interested in _package statements after point.
  (save-match-data
    (if (re-search-backward "^\\s-*\\(_package \\w+\\)\\s-*$" nil t)
	(concat (buffer-substring-no-properties
		 (match-beginning 1) (match-end 1))
		"\n"))))

(defun magik-transmit-string (str package do-magik-command tidy-magik-command &optional start gis process)
   "Generalised function to send code to Magik via a temporary file.
If this command is repeated before the previous file has been processed by Magik,
another file shall be written."
   (let* ((gis (sw-get-buffer-mode gis
				   'gis-mode
				   resources-gis-enter-buffer
				   gis-buffer
				   'gis-buffer-alist-prefix-function))
	  (process (barf-if-no-gis gis process))
	  (orig-buf  (buffer-name))
	  (orig-file (or (buffer-file-name) ""))
	  (position  (if start (number-to-string start) "1"))
	  (filename (concat (if (running-under-nt)
				(concat (getenv "TEMP") "\\T")
			      "/tmp/t")
			    (user-login-name)
			    (number-to-string (process-id process))))
         (package (or package "\n")) ;need a newline to ensure fixed number of lines for gis-goto-error
	 (coding-system buffer-file-coding-system))

     (setq filename (loop
		     with queue = 0
		     with file  = nil
		     do (setq file (concat filename "q" (number-to-string queue)))
		     if (file-exists-p file)
		     do (setq queue (1+ queue))
		     else
		     return file))
     (save-excursion
       (set-buffer (get-buffer-create " *transmit magik debug*"))
       (erase-buffer)
       (setq buffer-file-coding-system coding-system)
       (insert "write(\"**** Emacs: buffer=" orig-buf
	                            " file=" orig-file
				" position=" position
	       "\")\n$\n"
	       package
	       str)
       (goto-char (point-min))
       (if transmit-magik-debug-p
           (magik-perform-replace-no-set-mark "#DEBUG" "" nil))
       (write-region (point-min) (point-max) filename nil 'xxx)
       ;(kill-buffer (current-buffer))
       )
     (message resources-magik-transmit-string gis)
     (process-send-string
      process
      (concat
       "_protect\n"
       (funcall do-magik-command filename)
       "_protection\n"
       (funcall tidy-magik-command filename)
       "_endprotect\n"
       "$\n"))
     gis))

(defun magik-gis-drag-n-drop-load (gis filename)
  "Interface to Drag 'n' Drop GIS mode.
Called by `gis-drag-n-drop-load' when a Magik file is dropped."
  (let ((process (barf-if-no-gis gis)))
    (message resources-magik-transmit-string gis)
    (process-send-string
     process
     (concat
      (magik-function "load_file" filename)
      "$\n"))))

(defun magik-mode ()
  "Major mode for editing Magik code.

Indents with the TAB or RET keys, inserts underscores, and sends Magik
to a running gis with `F2 b', `F2 m', `F2 r', or `F2 RET'.
Creates programming templates like
  _if
  _then

  _endif
with `F2 SPC' and trace statements with `F2 t'.
Fills private (#) or public (##) comments with `F2 q'.

You can customise magik-mode with the magik-mode-hook."

  (interactive)
  (kill-all-local-variables)
  (make-local-variable 'paragraph-start)
  (make-local-variable 'paragraph-separate)
  (make-local-variable 'indent-line-function)
  (make-local-variable 'require-final-newline)
  (make-local-variable 'comment-start)
  (make-local-variable 'comment-end)
  (make-local-variable 'comment-column)
  (make-local-variable 'comment-start-skip)
  (make-local-variable 'comment-multi-line)
  (make-local-variable 'parse-sexp-ignore-comments)
  (make-local-variable 'imenu-generic-expression)
  (make-local-variable 'imenu-syntax-alist)
  (make-local-variable 'font-lock-defaults)
  (make-local-variable 'outline-regexp)
  (make-local-variable 'magik-method-name)

  (use-local-map magik-mode-map)
  (easy-menu-add magik-menu)
  (set-syntax-table magik-mode-syntax-table)

  (setq major-mode 'magik-mode
	mode-name resources-magik-menu
	magik-template-file-type (magik-template-file-type)
	local-abbrev-table magik-mode-abbrev-table
	paragraph-start (concat "^$\\|" page-delimiter)
	paragraph-separate paragraph-start
	indent-line-function 'magik-indent-line
	require-final-newline t
	comment-start "#"
	comment-end ""
	comment-column 8
	comment-start-skip "#+ *"
	comment-multi-line nil
	parse-sexp-ignore-comments nil
	imenu-create-index-function 'magik-imenu-create-index-function
	imenu-syntax-alist '((?_ . "w"))
	font-lock-defaults
	'((magik-font-lock-keywords
	   magik-font-lock-keywords-1
	   magik-font-lock-keywords-2
	   magik-font-lock-keywords-3
	   magik-font-lock-keywords-4)
	  nil t
	  ((?_ . "w"))
	  magik-goto-code
	  (font-lock-fontify-buffer-function   . magik-font-lock-fontify-buffer)
	  (font-lock-fontify-region-function   . magik-font-lock-fontify-region)
	  (font-lock-unfontify-buffer-function . magik-font-lock-unfontify-buffer))
	outline-regexp "\\(^\\(_private +\\|\\)\\(_iter +\\|\\)_method.*\\|.*\.def_property\\|.*\.define_\\(shared_variable\\|shared_constant\\|slot_access\\|property\\|interface\\|method_signature\\).*\\|^\\(\t*#+\>[^>]\\|def_slotted_exemplar\\|def_mixin\\|#% text_encoding\\|_global\\|read_\\(message\\|translator\\)_patch\\).*\\)")

  (if magik-auto-abbrevs (abbrev-mode 1))

  (run-hooks 'magik-mode-hook))

(defun magik-method-name-mode (&optional arg)
  "Toggle display of current method in mode line.
Toggles the value of `magik-method-name-mode'.
With a positive numeric arg, display method name on mode line,
With a negative numeric arg, remove  method name from the mode line."
  (interactive "P")
  (setq magik-method-name-mode
	(if (null arg)
	    (not magik-method-name-mode)
	  (> (prefix-numeric-value arg) 0)))
  (dolist (buf (sw-buffer-mode-list 'magik-mode))
    (with-current-buffer buf
      (magik-method-name-set)
      (force-mode-line-update)))
  (if magik-method-name-mode
      (add-hook 'post-command-hook 'magik-method-name-set)
    (remove-hook 'post-command-hook 'magik-method-name-set))
  (message
   (if magik-method-name-mode
       resources-magik-method-name-on
     resources-magik-method-name-off)))

(defun magik-method-name-set-text-properties (buf method)
  "Return string combining BUF and METHOD suitable for display in mode-line."
  ;;propertize only on Emacs 21 so we use add-text-properties
  (add-text-properties 0 (length buf)
		       (list
			'face '(:weight bold)
			'help-echo
			(purecopy "mouse-1: previous buffer, mouse-3: next buffer")
			'local-map mode-line-buffer-identification-keymap)
		       buf)
  (add-text-properties 0 (length method)
		       (list
			'face '(:inverse-video t)
			'help-echo
			(purecopy "mouse-1: previous buffer, mouse-3: next buffer")
			'local-map mode-line-buffer-identification-keymap)
		       method)
  (concat buf method))

(defun magik-method-name-set ()
  "Sets the name of the current Magik method in `magik-method-name'."
  (cond ((not (eq major-mode 'magik-mode))
	 nil)
	((not magik-method-name-mode)
	 (setq mode-line-buffer-identification
		(default-value 'mode-line-buffer-identification)))
	(t
	 (let ((this-syntax-table (copy-syntax-table magik-mode-syntax-table))
	       (buf (format "%-12s " (buffer-name)))
	       name
	       pt end)
	   (condition-case err
	       (save-excursion
		 (setq name ""
		       mode-line-buffer-identification 'magik-method-name)
		 (save-match-data
		   (modify-syntax-entry ?_ "w" this-syntax-table)
		   (set-syntax-table this-syntax-table)
		   (cond ((save-excursion
			    ;;method creation methods, currently ignores pragma
			    (setq pt (point))
			    (and
			     (search-forward ")" nil t)
			     (search-backward "(" nil t)
			     (progn (beginning-of-line) t)
			     (<= (point) pt)
			     (looking-at "^\\s-*\\sw+\.def\\(ine_\\(slot_access\\|shared_constant\\|shared_variable\\|property\\)\\|_property\\)([ \t\n]*:\\s-*\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)")))
			  (setq name (match-string-no-properties 3)))
			 ((save-excursion
			    ;;arrays
			    (beginning-of-line)
			    (setq pt (point))
			    (and (forward-endmethod t)
				 (setq end (point))
				 (backward-method t)
				 (<= (point) pt)
				 (re-search-forward "^\\s-*_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\s-*?\\[" end t)))
			  (setq name (car (magik-imenu-method-name 1))))
			 ((save-excursion
			    ;;normal methods
			    (beginning-of-line)
			    (setq pt (point))
			    (and (forward-endmethod t)
				 (backward-method t)
				 (<= (point) pt)
				 (re-search-forward "^.*\\b_method\\s-+\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\.\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\(\\s-*\\^?<<\\)?" nil t)))
			  (setq name (car (magik-imenu-method-name 2))))
			 (t nil)))
		 (setq magik-method-name (funcall magik-method-name-set-text-function buf name)))
	     (error
	      (setq magik-method-name "")))
	   (set-syntax-table magik-mode-syntax-table)
	   magik-method-name))))

(defun magik-trace-curr-statement ()
  "Add a trace statement for the current statement."
  (interactive "*")
  (back-to-indentation)
  (let
      ((col (current-column))
       (str (gsub (buffer-substring-no-properties
		   (point) (point-eol)) "\"" "\" + %\" + \"")))
    (beginning-of-line)
    (indent-to col)
    (insert "write(\""
            ;; (make-string col ? ) ;;;; withdrawn.
            "+++ "
            str
            " +++\")\n")))



(defun magik-heading ()
  "Transforms a Magik comment line written conventionally into a standard heading format.
   The format is # (or ##) <tab> followed by each character uppercased and single spaced.
   The body of the text is followed by a underline using # or ## as appropriate"
  (interactive "*")
  (save-excursion
    (save-restriction
      (narrow-to-region (point-bol) (point-eol))
      (upcase-region (point-bol) (point-eol))
      (beginning-of-line)
      (forward-char 2)
      (delete-horizontal-space)
      (replace-regexp "\\(.\\)" " \\1")
      (end-of-line)
      (insert "\n" (buffer-substring-no-properties (point-bol) (point-eol)))
      (beginning-of-line)
      (replace-regexp "[^#]" "_")
      (forward-line -1)
      (replace-regexp "^\\(##?\\)." "\\1\t"))))

(defun fill-magik-public-comment ()
  "Fill a comment paragraph."
  (interactive "*")
  (save-excursion
    (if (progn
          (back-to-indentation)
          (looking-at "\\(##?\\)[ \t]+[^ \t\n]"))
	(let*
	    ((comment-str (match-string 1))
	     (regexp-str (concat
			  "[ \t]*"
			  comment-str
			  "[ \t]+[^ \t\n]"))
	     beg
	     (fill-prefix
	      (concat (progn
			(back-to-indentation)
			(buffer-substring-no-properties (point-bol) (point)))
		      comment-str
		      " "))
	     (fill-column (+ (current-column) 63)))
	  (beginning-of-line)
	  (while
	      (and (looking-at regexp-str)
		   (zerop (forward-line -1))))
	  (if (not (looking-at regexp-str))
	      (forward-line 1))
	  (setq beg (point))
	  (while
	      (and (looking-at regexp-str)
		   (zerop (forward-line 1))))
	  (fill-region-as-paragraph beg (point))))))

;;; Mods to do commenting and uncommenting in magik code (Sarfaraz).
;; AJM: TODO Use comment-dwim what about Emacs 19 and 20???
(defun sw:comment (nlines)
"Puts # in first column of ARG lines at point"
  (interactive "p")
    (while (and (> nlines 0) (not (eobp)))
       (decf nlines)
       (beginning-of-line 1)
       (insert-char ?# 1)
       (forward-line 1)))

(defun sw:un-comment (nlines)
"Deletes # in first column of ARG lines at point"
  (interactive "*p")
    (while (and (> nlines 0) (not (eobp)))
       (decf nlines)
       (beginning-of-line 1)
       (if (char-equal (char-after (point)) ?# ) (delete-char 1))
       (forward-line 1)))

(defun magik-comment-region()
"Puts # in first column of each line in the region"
  (interactive "*")
  (save-excursion
    (if (> (point) (mark t)) (exchange-point-and-mark))
    (sw:comment (count-lines (point) (mark t)))))

(defun magik-uncomment-region()
"Removes # in first column of each line in the region"
  (interactive "*")
  (save-excursion
    (if (> (point) (mark t)) (exchange-point-and-mark))
    (sw:un-comment (count-lines (point) (mark t)))))

(defun magik-symbol-complete (&optional buffer)
  "Perform completion on Magik symbol preceding point.
The symbol is compared against the symbols that exist in the Magik
process running in the buffer named in the variable, `gis-buffer'.

With a prefix arg, ask user for GIS buffer to use."
  (interactive "*")
  ;; the actual completion is done by the process filter: gis-filter-completion-action
  (setq buffer (sw-get-buffer-mode buffer
				   'gis-mode
				   resources-gis-enter-buffer
				   gis-buffer
				   'gis-buffer-alist-prefix-function))
  (barf-if-no-gis buffer)

  (if (equal (sw-curr-word) "")
      (message resources-magik-complete-empty-string)
    (if (<= (length (sw-curr-word)) 2)
        (message resources-magik-complete-string))
    (process-send-string
     (get-buffer-process buffer)
     (concat "symbol_table.emacs_write_completions(\"" (sw-curr-word) "\")\n$\n"))))

(defun magik-compare-methods (ignore-whitespace)
  "Compare Methods in two windows using \\[compare-windows].

A prefix arg means ignore changes in whitespace.
The variable `compare-windows-whitespace' controls how whitespace is skipped.
If `compare-ignore-case' is non-nil, changes in case are also ignored."
  (interactive "P")
  (backward-method t)
  (push-mark nil t)
  (other-window 1)
  (backward-method t)
  (push-mark nil t)
  (other-window 1)
  (compare-windows ignore-whitespace))

(defun magik-ediff-methods ()
  "Compare Methods using Ediff in two windows using \\[ediff-regions-wordwise]."
  (interactive)
  (save-excursion
    (let ((buf-a (current-buffer))
	  buf-b)
      (magik-mark-method)
      (other-window 1)
      (setq buf-b (current-buffer))
      (magik-mark-method)
      (ediff-regions-wordwise buf-a buf-b))))

(defun magik-set-work-buffer-name (buffer)
  "Set `magik-work-buffer'."
  (interactive "BBuffer: ")
  (setq magik-work-buffer buffer)
  (save-excursion
    (set-buffer (get-buffer-create magik-work-buffer))
    (magik-mode)))

(defun magik-copy-method-to-buffer (&optional buffer)
  "Copy method to BUFFER.
 If BUFFER is nil, use buffer name stored in variable `magik-work-buffer'.
Otherwise create a sensibly named buffer based upon the class name of the method."
  (interactive)
  (save-excursion
    (save-match-data
      (forward-endmethod)
      (backward-method)
      (push-mark nil t)
      (while
	  (not (or (eobp)
		   (looking-at "^\\s-*\\(_abstract\\(\n\\|\\s-\\)+\\)?\\(_private\\(\n\\|\\s-\\)+\\)?\\(_iter\\(\n\\|\\s-\\)+\\)?_method\\s-+\\(\\sw*\\(\\s$\\S$*\\s$\\sw*\\)?\\)\\.")))
	(forward-line 1))
      (goto-char (match-end 0))
      (setq buffer (or buffer
		       magik-work-buffer
		       (concat (match-string-no-properties 7)
			       ".magik")))

      (cond ((eq (following-char) ?\()
	     (or (search-forward ")" nil t)
		 (error resources-magik-no-round-bracket-error)))
	    ((eq (following-char) ?\[)
	     (or (search-forward "]" nil t)
		 (error resources-magik-no-square-bracket-error))))

      (forward-endmethod)
      (magik-copy-region-to-buffer (mark t) (point) buffer))))

(defun magik-copy-region-to-buffer (start end &optional buffer)
  "Copy region to BUFFER.
 If BUFFER is nil, use buffer name stored in variable `magik-work-buffer'
otherwise prompt user."
  (interactive "r")
  (setq buffer (or buffer magik-work-buffer (read-buffer "Buffer: ")))

  (let ((code (buffer-substring start end)))
    (set-buffer (get-buffer-create buffer))
    (goto-char (point-max))
    (or (looking-at "^") (insert "\n"))
    (insert code "\n")
    (set-buffer-modified-p nil)
    (magik-mode))
  (display-buffer buffer))

;;; Imenu configuration functions
(defun magik-imenu-method-name (index)
  "Helper function for creating the correct name for a method in Imenu.
Used by `magik-imenu-expression' and `magik-imenu-create-index-function'
to return a syntactically correct method name for the Imenu menu entry."
  (let ((beg (match-beginning index))
	(name (match-string-no-properties index))
	postfix)
    (save-excursion
      (goto-char (match-end index))
      (skip-syntax-forward "-")
      (setq postfix
	    (cond ((eq (following-char) ?\() "()")
		  ((eq (following-char) ?<)  "<<")
		  ((looking-at "\\^<<")      "^<<")
		  ((looking-at "\\[[^\],]+,")
		   (if (re-search-forward "\\^?<<" (point-eol) t)
		       (concat "[,]" (match-string 0))
		     "[,]"))
		  ((eq (following-char) ?\[)
		   (if (re-search-forward "\\^?<<" (point-eol) t)
		       (concat "[]" (match-string 0))
		     "[]"))
		  (t "")))
      (cons (concat name postfix) beg))))

(defun magik-imenu-create-index-function ()
  "Return an index of the current buffer as an alist.

Uses `magik-imenu-expression' as the alist with elements that look like this:
 (MENU-TITLE REGEXP INDEX).
or
 (MENU-TITLE REGEXP FUNCTION ARGS).

MENU-TITLE is a string used as the title for the submenu or nil if the
entries are not nested.

REGEXP is a regexp that should match a construct in the buffer that is
to be displayed in the menu; i.e., function or variable definitions,
etc.  It contains a substring which is the name to appear in the
menu.  See the info section on Regexps for more information.

INDEX points to the substring in REGEXP that contains the name (of the
function, variable or type) that is to appear in the menu.

FUNCTION and ARGS are usedto generate the name based upon the result
of the regexp match.

Returns an index of the current buffer as an alist.  The elements in
the alist look like: (INDEX-NAME . INDEX-POSITION).  They may also be
nested index lists like (INDEX-NAME . INDEX-ALIST) depending on
PATTERNS.

This function is modified from `imenu--generic-function' to basically
provide extra control over the name that appears in the index."

  (let ((index-alist (list 'dummy))
	prev-pos beg
        (case-fold-search imenu-case-fold-search)
        (old-table (syntax-table))
        (table (copy-syntax-table (syntax-table)))
        (slist imenu-syntax-alist))
    ;; Modify the syntax table used while matching regexps.
    (while slist
      ;; The character(s) to modify may be a single char or a string.
      (if (numberp (caar slist))
	  (modify-syntax-entry (caar slist) (cdar slist) table)
	(mapc (lambda (c)
		(modify-syntax-entry c (cdar slist) table))
	      (caar slist)))
      (setq slist (cdr slist)))
    (goto-char (point-max))
    ;(imenu-progress-message prev-pos 0 t) ;LIG-deprecated in imenu.el
    (unwind-protect			; for syntax table
	(save-match-data
	  (set-syntax-table table)
	  ;; map over the elements of imenu-generic-expression
	  ;; (typically functions, variables ...)
	  (mapc
	   (lambda (pat)
	     (let ((menu-title (car pat))
		   (regexp (nth 1 pat))
		   (index (nth 2 pat))   ;index and function occupy 3 element
		   (function (nth 2 pat))
		   (rest (cdddr pat)))
	       ;; Go backwards for convenience of adding items in order.
	       (goto-char (point-max))
	       (while (re-search-backward regexp nil t)
		 ;(imenu-progress-message prev-pos nil t) ;LIG-deprecated in imenu.el
		 ;; Add this sort of submenu only when we've found an
		 ;; item for it, avoiding empty, duff menus.
		 (unless (assoc menu-title index-alist)
		   (push (list menu-title) index-alist))

		 ;; menu is the desired submenu,
		 ;; starting with its title (or nil).
		 (let ((menu (assoc menu-title index-alist))
		       item)
		   (if (functionp function)
		       (setq item (apply function rest))
		     ;;We have a simply index
		     (setq beg (match-beginning index))
		     (if imenu-use-markers
			 (setq beg (copy-marker beg)))
		     (setq item (cons (match-string-no-properties index)
				      beg)))
		   ;; Insert the item unless it is already present.
		   (unless (member item (cdr menu))
		     (setcdr menu
			     (cons item (cdr menu))))))))
	   magik-imenu-expression)
	  (set-syntax-table old-table)))
    ;(imenu-progress-message prev-pos 100 t) ;LIG-deprecated in imenu.el
    ;; Sort each submenu by position.
    ;; This is in case one submenu gets items from two different regexps.
    (let ((tail index-alist))
      (while tail
	(if (listp (car tail))
	    (setcdr (car tail)
		    (sort (cdr (car tail)) 'imenu--sort-by-position)))
	(setq tail (cdr tail))))
    (let ((main-element (assq nil index-alist)))
      (nconc (delq main-element (delq 'dummy index-alist))
	     (cdr main-element)))))

;;; Smallworld Compatibility functions
(defalias 'magik-point-on-pragma-line-p 'pragma-line-p)

(defun magik-translate-old-vec-notation ()
  "Search for the next \"vec(\" in the current buffer and translate it and the
closing bracket into the new \"{...}\" notation."
  (interactive)
  (re-search-forward "\\<vec(")
  (backward-char 4)
  (delete-char 3)
  (save-excursion
    (forward-sexp)
    (backward-delete-char 1)
    (insert "\}"))
  (delete-char 1)
  (insert "\{"))
(put 'magik-translate-old-vec-notation 'disabled t)

;;; Package initialisation
(if magik-mode-abbrev-table
    ()
  (define-abbrev-table 'magik-mode-abbrev-table ())
  (let ((abbrevs-changed nil))
    (mapcar
     '(lambda (str)
        (define-abbrev magik-mode-abbrev-table
          str str 'expand-magik-abbrev))

     (append magik-keyword-constants magik-keyword-operators
	     magik-keyword-class magik-keyword-statements
	     magik-keyword-methods magik-keyword-procedures
	     magik-keyword-loop magik-keyword-arguments
	     magik-keyword-variable nil))))

(if magik-mode-syntax-table
    ()
  (setq magik-mode-syntax-table (make-syntax-table))
  (if magik-under-as-char
      (modify-syntax-entry ?_ "w" magik-mode-syntax-table))
  (modify-syntax-entry ?\\ "." magik-mode-syntax-table) ;; \ is not an escape character in magik mode.
  (modify-syntax-entry ?? "w" magik-mode-syntax-table)
  (modify-syntax-entry ?! "w" magik-mode-syntax-table)
  ;; char intro
  (modify-syntax-entry ?% "/" magik-mode-syntax-table)
  ;; multi quote
  (modify-syntax-entry ?| "$" magik-mode-syntax-table)
  ;; colon is now part of a word due to the introduction of packages.
  ;;       Consequently symbols now include the initial :
  (modify-syntax-entry ?: "w" magik-mode-syntax-table) ;cf \ = "/" in TeX mode.
  ;; comments
  (modify-syntax-entry ?# "<" magik-mode-syntax-table)
  (modify-syntax-entry ?\n ">" magik-mode-syntax-table)
  (modify-syntax-entry ?+ "." magik-mode-syntax-table)
  (modify-syntax-entry ?- "." magik-mode-syntax-table)
  (modify-syntax-entry ?* "." magik-mode-syntax-table)
  (modify-syntax-entry ?/ "." magik-mode-syntax-table)
  (modify-syntax-entry ?= "." magik-mode-syntax-table)
  (modify-syntax-entry ?$ "." magik-mode-syntax-table)
  (modify-syntax-entry ?< "." magik-mode-syntax-table)
  (modify-syntax-entry ?> "." magik-mode-syntax-table)
  (modify-syntax-entry ?& "." magik-mode-syntax-table)
  (modify-syntax-entry ?\" "\"" magik-mode-syntax-table))

;;; package setup via setting of variable before load.
(and magik-method-name-mode
     (magik-method-name-mode magik-method-name-mode))
(and magik-work-buffer
     (magik-set-work-buffer-name magik-work-buffer))
(and magik-transmit-method-eom-mode
     (magik-transmit-method-eom-mode magik-transmit-method-eom-mode))

;;This functionality does not actually require a function to enable as it
;;purely controlled via the value of the variable magik-mark-method-exchange
;;(and magik-mark-method-exchange
;;     (magik-mark-method-exchange-mode magik-mark-method-exchange))

;; This function is defined in magik-patch minor mode but used in magik-mode-hook
;; We define it here in case a user stores magik-mode-hook in their .emacs and also
;; switches between the Emacs development environment setup and Emacs customer setup.
(or (functionp 'magik-patch-maybe-turn-on-patch-mode) ; only define it if undefined.
    (defalias 'magik-patch-maybe-turn-on-patch-mode 'ignore))

;;; Package registration

(or (assoc "\\.magik$" auto-mode-alist)
    (push '("\\.magik$" . magik-mode) auto-mode-alist))

;;; speedbar configuration
(if (featurep 'speedbar)
    (speedbar-add-supported-extension ".magik")
  (add-hook 'speedbar-load-hook
	    (function (lambda () (speedbar-add-supported-extension ".magik")))))
;;MSB configuration
(defun magik-msb-configuration ()
  "Adds Magik files to msb menu, supposes that msb is already loaded."
  (let* ((l (length msb-menu-cond))
	 (last (nth (1- l) msb-menu-cond))
	 (precdr (nthcdr (- l 2) msb-menu-cond)) ; cdr of this is last
	 (handle (1- (nth 1 last))))
    (setcdr precdr (list
		    (list
		     '(eq major-mode 'magik-mode)
		     handle
		     "Magik Files (%d)")
		    last))))

(if (featurep 'msb)
    (magik-msb-configuration)
  (add-hook 'msb-after-load-hook 'magik-msb-configuration))

(provide 'magik)
;;; magik.el ends here
